#!/usr/bin/env python
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example illustrates how to set up advanced remarketing.

Creates a rule-based user list defined by an expression rule for users who have
either checked out in November, or December, OR visited the checkout page with
more than one item in their cart.
"""

import argparse
import sys
from uuid import uuid4

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.common.types.user_lists import (
    FlexibleRuleOperandInfo,
    FlexibleRuleUserListInfo,
    RuleBasedUserListInfo,
    UserListRuleItemGroupInfo,
    UserListRuleItemInfo,
    UserListDateRuleItemInfo,
    UserListNumberRuleItemInfo,
    UserListStringRuleItemInfo,
)
from google.ads.googleads.v22.enums.types.user_list_date_rule_item_operator import (
    UserListDateRuleItemOperatorEnum,
)
from google.ads.googleads.v22.resources.types.user_list import UserList
from google.ads.googleads.v22.services.services.user_list_service import (
    UserListServiceClient,
)
from google.ads.googleads.v22.services.types.user_list_service import (
    MutateUserListsResponse,
    UserListOperation,
)


def main(client: GoogleAdsClient, customer_id: str) -> None:
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
    """
    # Get the UserListService client.
    user_list_service: UserListServiceClient = client.get_service(
        "UserListService"
    )

    # Creates the operation.
    user_list_operation: UserListOperation = client.get_type(
        "UserListOperation"
    )

    # Creates a user list.
    user_list: UserList = user_list_operation.create
    user_list.name = f"My expression rule user list #{uuid4()}"
    user_list.description = (
        "Users who checked out in November or December OR visited the "
        "checkout page with more than one item in their cart"
    )
    user_list.membership_status = client.enums.UserListMembershipStatusEnum.OPEN
    user_list.membership_life_span = 90

    # Defines a representation of a user list that is generated by a rule.
    rule_based_user_list_info: RuleBasedUserListInfo = (
        user_list.rule_based_user_list
    )
    rule_based_user_list_info.prepopulation_status = (
        client.enums.UserListPrepopulationStatusEnum.REQUESTED
    )

    # Creates a rule group that includes the checkout and cart size rules.
    # Combining the two rule items into a UserListRuleItemGroupInfo object
    # causes Google Ads to AND their rules together. To instead OR the rules
    # together, each rule should be placed in its own rule item group.
    checkout_and_cart_size_rule_group: UserListRuleItemGroupInfo = (
        client.get_type("UserListRuleItemGroupInfo")
    )
    # Creates a rule targeting any user that visited the checkout page.
    # [START setup_advanced_remarketing]
    checkout_rule: UserListRuleItemInfo = client.get_type(
        "UserListRuleItemInfo"
    )

    # The rule variable name must match a corresponding key name fired from a
    # pixel. To learn more about setting up remarketing tags, visit:
    # https://support.google.com/google-ads/answer/2476688.
    #
    # To learn more about remarketing events and parameters, visit:
    # https://support.google.com/google-ads/answer/7305793.
    checkout_rule.name = "ecomm_pagetype"
    checkout_string_rule_item: UserListStringRuleItemInfo = (
        checkout_rule.string_rule_item
    )
    checkout_string_rule_item.operator = (
        client.enums.UserListStringRuleItemOperatorEnum.EQUALS
    )
    checkout_string_rule_item.value = "checkout"
    # [END setup_advanced_remarketing]

    # Creates a rule targeting any user that had more than one item in their
    # cart.
    # [START setup_advanced_remarketing_1]
    cart_size_rule: UserListRuleItemInfo = client.get_type(
        "UserListRuleItemInfo"
    )
    # The rule variable name must match a corresponding key name fired from a
    # pixel.
    cart_size_rule.name = "cart_size"
    cart_size_number_rule_item: UserListNumberRuleItemInfo = (
        cart_size_rule.number_rule_item
    )
    cart_size_number_rule_item.operator = (
        client.enums.UserListNumberRuleItemOperatorEnum.GREATER_THAN
    )
    cart_size_number_rule_item.value = 1.0
    # [END setup_advanced_remarketing_1]

    # Creates a rule group targeting users who checked out between November
    # and December by using the start and end date rules. Combining the two
    # rule items into a UserListRuleItemGroupInfo object causes Google Ads to
    # AND their rules together. To instead OR the rules together, each rule
    # should be placed in its own rule item group.
    checkout_date_rule_group: UserListRuleItemGroupInfo = client.get_type(
        "UserListRuleItemGroupInfo"
    )

    # Creates the rule item for checkout start date.
    # The tags and keys used below must have been in place in the past for the
    # date range specified in the rules.
    # [START setup_advanced_remarketing_3]
    start_date_rule: UserListRuleItemInfo = client.get_type(
        "UserListRuleItemInfo"
    )
    start_date_rule.name = "checkoutdate"
    start_date_rule_item: UserListDateRuleItemInfo = (
        start_date_rule.date_rule_item
    )
    # Available UserListDateRuleItemOperators can be found at:
    # https://developers.google.com/google-ads/api/reference/rpc/latest/UserListDateRuleItemOperatorEnum.UserListDateRuleItemOperator
    user_list_data_rule_item_operator_enum: UserListDateRuleItemOperatorEnum = (
        client.enums.UserListDateRuleItemOperatorEnum
    )
    start_date_rule_item.operator = user_list_data_rule_item_operator_enum.AFTER
    start_date_rule_item.value = "20191031"
    # [END setup_advanced_remarketing_3]

    # Creates the rule item for checkout end date.
    # [START setup_advanced_remarketing_4]
    end_date_rule: UserListRuleItemInfo = client.get_type(
        "UserListRuleItemInfo"
    )
    end_date_rule.name = "checkoutdate"
    end_date_rule_item: UserListDateRuleItemInfo = end_date_rule.date_rule_item
    end_date_rule_item.operator = user_list_data_rule_item_operator_enum.BEFORE
    end_date_rule_item.value = "20200101"
    # [END setup_advanced_remarketing_4]

    # [START setup_advanced_remarketing_2]
    checkout_and_cart_size_rule_group.rule_items.extend(
        [
            checkout_rule,
            cart_size_rule,
        ]
    )
    # [END setup_advanced_remarketing_2]
    # [START setup_advanced_remarketing_5]
    checkout_date_rule_group.rule_items.extend(
        [
            start_date_rule,
            end_date_rule,
        ]
    )
    # [END setup_advanced_remarketing_5]

    # [START setup_advanced_remarketing_6]
    # Create a FlexibleRuleUserListInfo object, or a flexible rule
    # representation of visitors with one or multiple actions.
    # FlexibleRuleUserListInfo wraps UserListRuleInfo in a
    # FlexibleRuleOperandInfo object that represents which user lists to
    # include or exclude.
    flexible_rule_user_list_info: FlexibleRuleUserListInfo = (
        rule_based_user_list_info.flexible_rule_user_list
    )
    flexible_rule_user_list_info.inclusive_rule_operator = (
        client.enums.UserListFlexibleRuleOperatorEnum.AND
    )
    # The default rule_type for a UserListRuleInfo object is OR of
    # ANDs (disjunctive normal form). That is, rule items will be
    # ANDed together within rule item groups and the groups
    # themselves will be ORed together.
    rule_operand: FlexibleRuleOperandInfo = client.get_type(
        "FlexibleRuleOperandInfo"
    )
    rule_operand.rule.rule_item_groups.extend(
        [
            checkout_and_cart_size_rule_group,
            checkout_date_rule_group,
        ]
    )
    rule_operand.lookback_window_days = 7
    flexible_rule_user_list_info.inclusive_operands.append(rule_operand)
    # [END setup_advanced_remarketing_6]

    # Issue a mutate request to add the user list, then print the results.
    response: MutateUserListsResponse = user_list_service.mutate_user_lists(
        customer_id=customer_id, operations=[user_list_operation]
    )
    print(
        "Created user list with resource name "
        f"'{response.results[0].resource_name}.'"
    )


if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description="Creates a rule-based user list defined by an expression "
        "rule for users who have either checked out in November, "
        "or December, OR visited the checkout page with more than "
        "one item in their cart."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(googleads_client, args.customer_id)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.enums.types import (
    keyword_plan_network as gage_keyword_plan_network,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "KeywordPlanCampaign",
        "KeywordPlanGeoTarget",
    },
)


class KeywordPlanCampaign(proto.Message):
    r"""A Keyword Plan campaign.
    Max number of keyword plan campaigns per plan allowed: 1.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the Keyword Plan campaign.
            KeywordPlanCampaign resource names have the form:

            ``customers/{customer_id}/keywordPlanCampaigns/{kp_campaign_id}``
        keyword_plan (str):
            The keyword plan this campaign belongs to.

            This field is a member of `oneof`_ ``_keyword_plan``.
        id (int):
            Output only. The ID of the Keyword Plan
            campaign.

            This field is a member of `oneof`_ ``_id``.
        name (str):
            The name of the Keyword Plan campaign.

            This field is required and should not be empty
            when creating Keyword Plan campaigns.

            This field is a member of `oneof`_ ``_name``.
        language_constants (MutableSequence[str]):
            The languages targeted for the Keyword Plan
            campaign. Max allowed: 1.
        keyword_plan_network (google.ads.googleads.v22.enums.types.KeywordPlanNetworkEnum.KeywordPlanNetwork):
            Targeting network.

            This field is required and should not be empty
            when creating Keyword Plan campaigns.
        cpc_bid_micros (int):
            A default max cpc bid in micros, and in the
            account currency, for all ad groups under the
            campaign.

            This field is required and should not be empty
            when creating Keyword Plan campaigns.

            This field is a member of `oneof`_ ``_cpc_bid_micros``.
        geo_targets (MutableSequence[google.ads.googleads.v22.resources.types.KeywordPlanGeoTarget]):
            The geo targets.
            Max number allowed: 20.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    keyword_plan: str = proto.Field(
        proto.STRING,
        number=9,
        optional=True,
    )
    id: int = proto.Field(
        proto.INT64,
        number=10,
        optional=True,
    )
    name: str = proto.Field(
        proto.STRING,
        number=11,
        optional=True,
    )
    language_constants: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=12,
    )
    keyword_plan_network: (
        gage_keyword_plan_network.KeywordPlanNetworkEnum.KeywordPlanNetwork
    ) = proto.Field(
        proto.ENUM,
        number=6,
        enum=gage_keyword_plan_network.KeywordPlanNetworkEnum.KeywordPlanNetwork,
    )
    cpc_bid_micros: int = proto.Field(
        proto.INT64,
        number=13,
        optional=True,
    )
    geo_targets: MutableSequence["KeywordPlanGeoTarget"] = proto.RepeatedField(
        proto.MESSAGE,
        number=8,
        message="KeywordPlanGeoTarget",
    )


class KeywordPlanGeoTarget(proto.Message):
    r"""A geo target.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        geo_target_constant (str):
            Required. The resource name of the geo
            target.

            This field is a member of `oneof`_ ``_geo_target_constant``.
    """

    geo_target_constant: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
This example shows how to create a complete Responsive Search ad.

Includes creation of: budget, campaign, ad group, ad group ad,
keywords, and geo targeting.

More details on Responsive Search ads can be found here:
https://support.google.com/google-ads/answer/7684791
"""

import argparse
import sys
import uuid
from typing import List, Optional

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.common.types.ad_asset import AdTextAsset
from google.ads.googleads.v22.enums.types.served_asset_field_type import (
    ServedAssetFieldTypeEnum,
)
from google.ads.googleads.v22.resources.types.ad_group import AdGroup
from google.ads.googleads.v22.resources.types.ad_group_ad import AdGroupAd
from google.ads.googleads.v22.resources.types.ad_group_criterion import (
    AdGroupCriterion,
)
from google.ads.googleads.v22.resources.types.campaign import Campaign
from google.ads.googleads.v22.resources.types.campaign_budget import (
    CampaignBudget,
)
from google.ads.googleads.v22.resources.types.campaign_criterion import (
    CampaignCriterion,
)
from google.ads.googleads.v22.resources.types.customer_customizer import (
    CustomerCustomizer,
)
from google.ads.googleads.v22.resources.types.customizer_attribute import (
    CustomizerAttribute,
)
from google.ads.googleads.v22.services.services.ad_group_ad_service import (
    AdGroupAdServiceClient,
)
from google.ads.googleads.v22.services.services.ad_group_criterion_service import (
    AdGroupCriterionServiceClient,
)
from google.ads.googleads.v22.services.services.ad_group_service import (
    AdGroupServiceClient,
)
from google.ads.googleads.v22.services.services.campaign_budget_service import (
    CampaignBudgetServiceClient,
)
from google.ads.googleads.v22.services.services.campaign_criterion_service import (
    CampaignCriterionServiceClient,
)
from google.ads.googleads.v22.services.services.campaign_service import (
    CampaignServiceClient,
)
from google.ads.googleads.v22.services.services.customer_customizer_service import (
    CustomerCustomizerServiceClient,
)
from google.ads.googleads.v22.services.services.customizer_attribute_service import (
    CustomizerAttributeServiceClient,
)
from google.ads.googleads.v22.services.services.geo_target_constant_service import (
    GeoTargetConstantServiceClient,
)
from google.ads.googleads.v22.services.types.geo_target_constant_service import (
    SuggestGeoTargetConstantsRequest,
    SuggestGeoTargetConstantsResponse,
)
from google.ads.googleads.v22.services.types.ad_group_ad_service import (
    AdGroupAdOperation,
    MutateAdGroupAdsResponse,
)
from google.ads.googleads.v22.services.types.ad_group_criterion_service import (
    AdGroupCriterionOperation,
    MutateAdGroupCriteriaResponse,
)
from google.ads.googleads.v22.services.types.ad_group_service import (
    AdGroupOperation,
    MutateAdGroupsResponse,
)
from google.ads.googleads.v22.services.types.campaign_budget_service import (
    CampaignBudgetOperation,
    MutateCampaignBudgetsResponse,
)
from google.ads.googleads.v22.services.types.campaign_criterion_service import (
    CampaignCriterionOperation,
    MutateCampaignCriteriaResponse,
)
from google.ads.googleads.v22.services.types.campaign_service import (
    CampaignOperation,
    MutateCampaignsResponse,
)
from google.ads.googleads.v22.services.types.customer_customizer_service import (
    CustomerCustomizerOperation,
    MutateCustomerCustomizersResponse,
)
from google.ads.googleads.v22.services.types.customizer_attribute_service import (
    CustomizerAttributeOperation,
    MutateCustomizerAttributesResponse,
)

# Keywords from user.
KEYWORD_TEXT_EXACT = "example of exact match"
KEYWORD_TEXT_PHRASE = "example of phrase match"
KEYWORD_TEXT_BROAD = "example of broad match"

# Geo targeting from user.
GEO_LOCATION_1 = "Buenos aires"
GEO_LOCATION_2 = "San Isidro"
GEO_LOCATION_3 = "Mar del Plata"

# LOCALE and COUNTRY_CODE are used for geo targeting.
# LOCALE is using ISO 639-1 format. If an invalid LOCALE is given,
# 'es' is used by default.
LOCALE = "es"

# A list of country codes can be referenced here:
# https://developers.google.com/google-ads/api/reference/data/geotargets
COUNTRY_CODE = "AR"


def main(
    client: GoogleAdsClient,
    customer_id: str,
    customizer_attribute_name: Optional[str] = None,
) -> None:
    """
    The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        customizer_attribute_name: The name of the customizer attribute to be
            created
    """
    if customizer_attribute_name:
        customizer_attribute_resource_name: str = create_customizer_attribute(
            client, customer_id, customizer_attribute_name
        )

        link_customizer_attribute_to_customer(
            client, customer_id, customizer_attribute_resource_name
        )

    # Create a budget, which can be shared by multiple campaigns.
    campaign_budget: str = create_campaign_budget(client, customer_id)

    campaign_resource_name: str = create_campaign(
        client, customer_id, campaign_budget
    )

    ad_group_resource_name: str = create_ad_group(
        client, customer_id, campaign_resource_name
    )

    create_ad_group_ad(
        client, customer_id, ad_group_resource_name, customizer_attribute_name
    )

    add_keywords(client, customer_id, ad_group_resource_name)

    add_geo_targeting(client, customer_id, campaign_resource_name)


def create_customizer_attribute(
    client: GoogleAdsClient, customer_id: str, customizer_attribute_name: str
) -> str:
    """Creates a customizer attribute with the given customizer attribute name.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        customizer_attribute_name: the name for the customizer attribute.

    Returns:
        A resource name for a customizer attribute.
    """
    # Create a customizer attribute operation for creating a customizer
    # attribute.
    operation: CustomizerAttributeOperation = client.get_type(
        "CustomizerAttributeOperation"
    )
    # Create a customizer attribute with the specified name.
    customizer_attribute: CustomizerAttribute = operation.create
    customizer_attribute.name = customizer_attribute_name
    # Specify the type to be 'PRICE' so that we can dynamically customize the
    # part of the ad's description that is a price of a product/service we
    # advertise.
    customizer_attribute.type_ = client.enums.CustomizerAttributeTypeEnum.PRICE

    # Issue a mutate request to add the customizer attribute and prints its
    # information.
    customizer_attribute_service: CustomizerAttributeServiceClient = (
        client.get_service("CustomizerAttributeService")
    )
    response: MutateCustomizerAttributesResponse = (
        customizer_attribute_service.mutate_customizer_attributes(
            customer_id=customer_id, operations=[operation]
        )
    )
    resource_name: str = response.results[0].resource_name

    print(f"Added a customizer attribute with resource name: '{resource_name}'")

    return resource_name


def link_customizer_attribute_to_customer(
    client: GoogleAdsClient,
    customer_id: str,
    customizer_attribute_resource_name: str,
) -> None:
    """Links the customizer attribute to the customer.

    Args:
        client: an initialized GoogleAdsClient instance.
            customer_id: a client customer ID.
        customizer_attribute_resource_name: a resource name for  customizer
            attribute.
    """
    # Create a customer customizer operation.
    operation: CustomerCustomizerOperation = client.get_type(
        "CustomerCustomizerOperation"
    )
    # Create a customer customizer with the value to be used in the responsive
    # search ad.
    customer_customizer: CustomerCustomizer = operation.create
    customer_customizer.customizer_attribute = (
        customizer_attribute_resource_name
    )
    customer_customizer.value.type_ = (
        client.enums.CustomizerAttributeTypeEnum.PRICE
    )
    # The ad customizer will dynamically replace the placeholder with this value
    # when the ad serves.
    customer_customizer.value.string_value = "100USD"

    customer_customizer_service: CustomerCustomizerServiceClient = (
        client.get_service("CustomerCustomizerService")
    )
    # Issue a mutate request to create the customer customizer and prints its
    # information.
    response: MutateCustomerCustomizersResponse = (
        customer_customizer_service.mutate_customer_customizers(
            customer_id=customer_id, operations=[operation]
        )
    )
    resource_name: str = response.results[0].resource_name

    print(
        f"Added a customer customizer to the customer with resource name: '{resource_name}'"
    )


def create_ad_text_asset(
    client: GoogleAdsClient,
    text: str,
    pinned_field: Optional[
        ServedAssetFieldTypeEnum.ServedAssetFieldType
    ] = None,
) -> AdTextAsset:
    """Create an AdTextAsset.

    Args:
        client: an initialized GoogleAdsClient instance.
        text: text for headlines and descriptions.
        pinned_field: to pin a text asset so it always shows in the ad.

    Returns:
        An AdTextAsset.
    """
    ad_text_asset: AdTextAsset = client.get_type("AdTextAsset")
    ad_text_asset.text = text
    if pinned_field:
        ad_text_asset.pinned_field = pinned_field
    return ad_text_asset


def create_ad_text_asset_with_customizer(
    client: GoogleAdsClient, customizer_attribute_resource_name: str
) -> AdTextAsset:
    """Create an AdTextAsset.
    Args:
        client: an initialized GoogleAdsClient instance.
        customizer_attribute_resource_name: The resource name of the customizer attribute.

    Returns:
        An AdTextAsset.
    """
    ad_text_asset: AdTextAsset = client.get_type("AdTextAsset")

    # Create this particular description using the ad customizer. Visit
    # https://developers.google.com/google-ads/api/docs/ads/customize-responsive-search-ads#ad_customizers_in_responsive_search_ads
    # for details about the placeholder format. The ad customizer replaces the
    # placeholder with the value we previously created and linked to the
    # customer using CustomerCustomizer.
    ad_text_asset.text = (
        f"Just {{CUSTOMIZER.{customizer_attribute_resource_name}:10USD}}"
    )

    return ad_text_asset


def create_campaign_budget(client: GoogleAdsClient, customer_id: str) -> str:
    """Creates campaign budget resource.

    Args:
      client: an initialized GoogleAdsClient instance.
      customer_id: a client customer ID.

    Returns:
      Campaign budget resource name.
    """
    # Create a budget, which can be shared by multiple campaigns.
    campaign_budget_service: CampaignBudgetServiceClient = client.get_service(
        "CampaignBudgetService"
    )
    campaign_budget_operation: CampaignBudgetOperation = client.get_type(
        "CampaignBudgetOperation"
    )
    campaign_budget: CampaignBudget = campaign_budget_operation.create
    campaign_budget.name = f"Campaign budget {uuid.uuid4()}"
    campaign_budget.delivery_method = (
        client.enums.BudgetDeliveryMethodEnum.STANDARD
    )
    campaign_budget.amount_micros = 500000

    # Add budget.
    campaign_budget_response: MutateCampaignBudgetsResponse = (
        campaign_budget_service.mutate_campaign_budgets(
            customer_id=customer_id, operations=[campaign_budget_operation]
        )
    )

    return campaign_budget_response.results[0].resource_name


def create_campaign(
    client: GoogleAdsClient, customer_id: str, campaign_budget: str
) -> str:
    """Creates campaign resource.

    Args:
      client: an initialized GoogleAdsClient instance.
      customer_id: a client customer ID.
      campaign_budget: a budget resource name.

    Returns:
      Campaign resource name.
    """
    campaign_service: CampaignServiceClient = client.get_service(
        "CampaignService"
    )
    campaign_operation: CampaignOperation = client.get_type("CampaignOperation")
    campaign: Campaign = campaign_operation.create
    campaign.name = f"Testing RSA via API {uuid.uuid4()}"
    campaign.advertising_channel_type = (
        client.enums.AdvertisingChannelTypeEnum.SEARCH
    )

    # Recommendation: Set the campaign to PAUSED when creating it to prevent
    # the ads from immediately serving. Set to ENABLED once you've added
    # targeting and the ads are ready to serve.
    campaign.status = client.enums.CampaignStatusEnum.PAUSED

    # Set the bidding strategy and budget.
    # The bidding strategy for Maximize Clicks is TargetSpend.
    # The target_spend_micros is deprecated so don't put any value.
    # See other bidding strategies you can select in the link below.
    # https://developers.google.com/google-ads/api/reference/rpc/latest/Campaign#campaign_bidding_strategy
    campaign.target_spend.target_spend_micros = 0
    campaign.campaign_budget = campaign_budget

    # Set the campaign network options.
    campaign.network_settings.target_google_search = True
    campaign.network_settings.target_search_network = True
    campaign.network_settings.target_partner_search_network = False
    # Enable Display Expansion on Search campaigns. For more details see:
    # https://support.google.com/google-ads/answer/7193800
    campaign.network_settings.target_content_network = True

    # # Optional: Set the start date.
    # start_time = datetime.date.today() + datetime.timedelta(days=1)
    # campaign.start_date = datetime.date.strftime(start_time, _DATE_FORMAT)

    # # Optional: Set the end date.
    # end_time = start_time + datetime.timedelta(weeks=4)
    # campaign.end_date = datetime.date.strftime(end_time, _DATE_FORMAT)

    # Add the campaign.
    campaign_response: MutateCampaignsResponse = (
        campaign_service.mutate_campaigns(
            customer_id=customer_id, operations=[campaign_operation]
        )
    )
    resource_name: str = campaign_response.results[0].resource_name
    print(f"Created campaign {resource_name}.")
    return resource_name


def create_ad_group(
    client: GoogleAdsClient,
    customer_id: str,
    campaign_resource_name: str,
) -> str:
    """Creates ad group.

    Args:
      client: an initialized GoogleAdsClient instance.
      customer_id: a client customer ID.
      campaign_resource_name: a campaign resource name.

    Returns:
      Ad group ID.
    """
    ad_group_service: AdGroupServiceClient = client.get_service(
        "AdGroupService"
    )

    ad_group_operation: AdGroupOperation = client.get_type("AdGroupOperation")
    ad_group: AdGroup = ad_group_operation.create
    ad_group.name = f"Testing RSA via API {uuid.uuid4()}"
    ad_group.status = client.enums.AdGroupStatusEnum.ENABLED
    ad_group.campaign = campaign_resource_name
    ad_group.type_ = client.enums.AdGroupTypeEnum.SEARCH_STANDARD

    # If you want to set up a max CPC bid uncomment line below.
    # ad_group.cpc_bid_micros = 10000000

    # Add the ad group.
    ad_group_response: MutateAdGroupsResponse = (
        ad_group_service.mutate_ad_groups(
            customer_id=customer_id, operations=[ad_group_operation]
        )
    )
    ad_group_resource_name: str = ad_group_response.results[0].resource_name
    print(f"Created ad group {ad_group_resource_name}.")
    return ad_group_resource_name


def create_ad_group_ad(
    client: GoogleAdsClient,
    customer_id: str,
    ad_group_resource_name: str,
    customizer_attribute_name: Optional[str],
) -> None:
    """Creates ad group ad.

    Args:
      client: an initialized GoogleAdsClient instance.
      customer_id: a client customer ID.
      ad_group_resource_name: an ad group resource name.
      customizer_attribute_name: (optional) If present, indicates the resource
        name of the customizer attribute to use in one of the descriptions

    Returns:
      None.
    """
    ad_group_ad_service: AdGroupAdServiceClient = client.get_service(
        "AdGroupAdService"
    )

    ad_group_ad_operation: AdGroupAdOperation = client.get_type(
        "AdGroupAdOperation"
    )
    ad_group_ad: AdGroupAd = ad_group_ad_operation.create
    ad_group_ad.status = client.enums.AdGroupAdStatusEnum.ENABLED
    ad_group_ad.ad_group = ad_group_resource_name

    # Set responsive search ad info.
    # https://developers.google.com/google-ads/api/reference/rpc/latest/ResponsiveSearchAdInfo

    # The list of possible final URLs after all cross-domain redirects for the ad.
    ad_group_ad.ad.final_urls.append("https://www.example.com/")

    # Set a pinning to always choose this asset for HEADLINE_1. Pinning is
    # optional; if no pinning is set, then headlines and descriptions will be
    # rotated and the ones that perform best will be used more often.

    # Headline 1
    served_asset_enum: ServedAssetFieldTypeEnum = (
        client.enums.ServedAssetFieldTypeEnum
    )
    pinned_headline: AdTextAsset = create_ad_text_asset(
        client, "Headline 1 testing", served_asset_enum
    )

    # Headline 2 and 3
    ad_group_ad.ad.responsive_search_ad.headlines.extend(
        [
            pinned_headline,
            create_ad_text_asset(client, "Headline 2 testing"),
            create_ad_text_asset(client, "Headline 3 testing"),
        ]
    )

    # Description 1 and 2
    description_1: AdTextAsset = create_ad_text_asset(client, "Desc 1 testing")
    description_2: Optional[AdTextAsset] = None

    if customizer_attribute_name:
        description_2 = create_ad_text_asset_with_customizer(
            client, customizer_attribute_name
        )
    else:
        description_2 = create_ad_text_asset(client, "Desc 2 testing")

    ad_group_ad.ad.responsive_search_ad.descriptions.extend(
        [description_1, description_2]
    )

    # Paths
    # First and second part of text that can be appended to the URL in the ad.
    # If you use the examples below, the ad will show
    # https://www.example.com/all-inclusive/deals
    ad_group_ad.ad.responsive_search_ad.path1 = "all-inclusive"
    ad_group_ad.ad.responsive_search_ad.path2 = "deals"

    # Send a request to the server to add a responsive search ad.
    ad_group_ad_response: MutateAdGroupAdsResponse = (
        ad_group_ad_service.mutate_ad_group_ads(
            customer_id=customer_id, operations=[ad_group_ad_operation]
        )
    )

    for result in ad_group_ad_response.results:
        print(
            f"Created responsive search ad with resource name "
            f'"{result.resource_name}".'
        )


def add_keywords(
    client: GoogleAdsClient, customer_id: str, ad_group_resource_name: str
) -> None:
    """Creates keywords.

    Creates 3 keyword match types: EXACT, PHRASE, and BROAD.

    EXACT: ads may show on searches that ARE the same meaning as your keyword.
    PHRASE: ads may show on searches that INCLUDE the meaning of your keyword.
    BROAD: ads may show on searches that RELATE to your keyword.
    For smart bidding, BROAD is the recommended one.

    Args:
      client: an initialized GoogleAdsClient instance.
      customer_id: a client customer ID.
      ad_group_resource_name: an ad group resource name.
    """
    ad_group_criterion_service: AdGroupCriterionServiceClient = (
        client.get_service("AdGroupCriterionService")
    )

    operations: List[AdGroupCriterionOperation] = []
    # Create keyword 1.
    ad_group_criterion_operation: AdGroupCriterionOperation = client.get_type(
        "AdGroupCriterionOperation"
    )
    ad_group_criterion: AdGroupCriterion = ad_group_criterion_operation.create
    ad_group_criterion.ad_group = ad_group_resource_name
    ad_group_criterion.status = client.enums.AdGroupCriterionStatusEnum.ENABLED
    ad_group_criterion.keyword.text = KEYWORD_TEXT_EXACT
    ad_group_criterion.keyword.match_type = (
        client.enums.KeywordMatchTypeEnum.EXACT
    )

    # Uncomment the below line if you want to change this keyword to a negative target.
    # ad_group_criterion.negative = True

    # Optional repeated field
    # ad_group_criterion.final_urls.append('https://www.example.com')

    # Add operation
    operations.append(ad_group_criterion_operation)

    # Create keyword 2.
    ad_group_criterion_operation: AdGroupCriterionOperation = client.get_type(
        "AdGroupCriterionOperation"
    )
    ad_group_criterion: AdGroupCriterion = ad_group_criterion_operation.create
    ad_group_criterion.ad_group = ad_group_resource_name
    ad_group_criterion.status = client.enums.AdGroupCriterionStatusEnum.ENABLED
    ad_group_criterion.keyword.text = KEYWORD_TEXT_PHRASE
    ad_group_criterion.keyword.match_type = (
        client.enums.KeywordMatchTypeEnum.PHRASE
    )

    # Uncomment the below line if you want to change this keyword to a negative target.
    # ad_group_criterion.negative = True

    # Optional repeated field
    # ad_group_criterion.final_urls.append('https://www.example.com')

    # Add operation
    operations.append(ad_group_criterion_operation)

    # Create keyword 3.
    ad_group_criterion_operation: AdGroupCriterionOperation = client.get_type(
        "AdGroupCriterionOperation"
    )
    ad_group_criterion: AdGroupCriterion = ad_group_criterion_operation.create
    ad_group_criterion.ad_group = ad_group_resource_name
    ad_group_criterion.status = client.enums.AdGroupCriterionStatusEnum.ENABLED
    ad_group_criterion.keyword.text = KEYWORD_TEXT_BROAD
    ad_group_criterion.keyword.match_type = (
        client.enums.KeywordMatchTypeEnum.BROAD
    )

    # Uncomment the below line if you want to change this keyword to a negative target.
    # ad_group_criterion.negative = True

    # Optional repeated field
    # ad_group_criterion.final_urls.append('https://www.example.com')

    # Add operation
    operations.append(ad_group_criterion_operation)

    # Add keywords
    ad_group_criterion_response: MutateAdGroupCriteriaResponse = (
        ad_group_criterion_service.mutate_ad_group_criteria(
            customer_id=customer_id,
            operations=operations,
        )
    )
    for result in ad_group_criterion_response.results:
        print("Created keyword " f"{result.resource_name}.")


def add_geo_targeting(
    client: GoogleAdsClient, customer_id: str, campaign_resource_name: str
) -> None:
    """Creates geo targets.

    Args:
      client: an initialized GoogleAdsClient instance.
      customer_id: a client customer ID.
      campaign_resource_name: an campaign resource name.

    Returns:
      None.
    """
    geo_target_constant_service: GeoTargetConstantServiceClient = (
        client.get_service("GeoTargetConstantService")
    )

    # Search by location names from
    # GeoTargetConstantService.suggest_geo_target_constants() and directly
    # apply GeoTargetConstant.resource_name.
    gtc_request: SuggestGeoTargetConstantsRequest = client.get_type(
        "SuggestGeoTargetConstantsRequest"
    )
    gtc_request.locale = LOCALE
    gtc_request.country_code = COUNTRY_CODE

    # The location names to get suggested geo target constants.
    gtc_request.location_names.names.extend(
        [GEO_LOCATION_1, GEO_LOCATION_2, GEO_LOCATION_3]
    )

    results: SuggestGeoTargetConstantsResponse = (
        geo_target_constant_service.suggest_geo_target_constants(gtc_request)
    )

    operations: List[CampaignCriterionOperation] = []
    for suggestion in results.geo_target_constant_suggestions:
        print(
            "geo_target_constant: "
            f"{suggestion.geo_target_constant.resource_name} "
            f"is found in LOCALE ({suggestion.locale}) "
            f"with reach ({suggestion.reach}) "
            f"from search term ({suggestion.search_term})."
        )
        # Create the campaign criterion for location targeting.
        campaign_criterion_operation: CampaignCriterionOperation = (
            client.get_type("CampaignCriterionOperation")
        )
        campaign_criterion: CampaignCriterion = (
            campaign_criterion_operation.create
        )
        campaign_criterion.campaign = campaign_resource_name
        campaign_criterion.location.geo_target_constant = (
            suggestion.geo_target_constant.resource_name
        )
        operations.append(campaign_criterion_operation)

    campaign_criterion_service: CampaignCriterionServiceClient = (
        client.get_service("CampaignCriterionService")
    )
    campaign_criterion_response: MutateCampaignCriteriaResponse = (
        campaign_criterion_service.mutate_campaign_criteria(
            customer_id=customer_id, operations=[*operations]
        )
    )

    for result in campaign_criterion_response.results:
        print(f'Added campaign criterion "{result.resource_name}".')


if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description=("Creates a Responsive Search Ad for specified customer.")
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )

    # The name of the customizer attribute used in the ad customizer, which
    # must be unique for a given customer account. To run this example multiple
    # times, specify a unique value as a command line argument. Note that there is
    # a limit for the number of enabled customizer attributes in one account
    # For more details visit:
    # https://developers.google.com/google-ads/api/docs/ads/customize-responsive-search-ads#rules_and_limitations
    parser.add_argument(
        "-n",
        "--customizer_attribute_name",
        type=str,
        default=None,
        help=(
            "The name of the customizer attribute to be created. The name must "
            "be unique across a client account, so be sure not to use "
            "the same value more than once."
        ),
    )

    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(
            googleads_client,
            args.customer_id,
            args.customizer_attribute_name,
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'Error with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



#!/usr/bin/env python
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Adds a shopping listing group tree to a shopping ad group.

The example will clear an existing listing group tree and rebuild it include the
following tree structure:

ProductCanonicalCondition NEW $0.20
ProductCanonicalCondition USED $0.10
ProductCanonicalCondition null (everything else)
    ProductBrand CoolBrand $0.90
    ProductBrand CheapBrand $0.01
    ProductBrand null (everything else) $0.50
"""

import argparse
import sys
from typing import List, Optional

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.common.types.criteria import (
    ListingDimensionInfo,
    ListingGroupInfo,
)
from google.ads.googleads.v22.enums.types.product_condition import (
    ProductConditionEnum,
)
from google.ads.googleads.v22.resources.types.ad_group_criterion import (
    AdGroupCriterion,
)
from google.ads.googleads.v22.services.services.ad_group_criterion_service import (
    AdGroupCriterionServiceClient,
)
from google.ads.googleads.v22.services.services.ad_group_service import (
    AdGroupServiceClient,
)
from google.ads.googleads.v22.services.services.google_ads_service import (
    GoogleAdsServiceClient,
)
from google.ads.googleads.v22.services.types.ad_group_criterion_service import (
    AdGroupCriterionOperation,
)
from google.ads.googleads.v22.services.types.google_ads_service import (
    MutateGoogleAdsResponse,
    SearchGoogleAdsResponse,
)

last_criterion_id: int = 0


def next_id() -> str:
    """Returns a decreasing negative number for temporary ad group criteria IDs.

    The ad group criteria will get real IDs when created on the server.
    Returns -1, -2, -3, etc. on subsequent calls.

    Returns:
        The string representation of a negative integer.
    """
    global last_criterion_id
    last_criterion_id -= 1
    return str(last_criterion_id)


# [START add_shopping_product_listing_group_tree]
def main(
    client: GoogleAdsClient,
    customer_id: str,
    ad_group_id: str,
    replace_existing_tree: bool,
) -> None:
    """Adds a shopping listing group tree to a shopping ad group.

    Args:
        client: An initialized Google Ads client.
        customer_id: The Google Ads customer ID.
        ad_group_id: The ad group ID to which the node will be added.
        replace_existing_tree: Boolean, whether to replace the existing listing
            group tree on the ad group. Defaults to false.
    """
    # Get the AdGroupCriterionService client.
    ad_group_criterion_service: AdGroupCriterionServiceClient = (
        client.get_service("AdGroupCriterionService")
    )

    # Optional: Remove the existing listing group tree, if it already exists
    # on the ad group. The example will throw a LISTING_GROUP_ALREADY_EXISTS
    # error if a listing group tree already exists and this option is not
    # set to true.
    if replace_existing_tree:
        remove_listing_group_tree(client, customer_id, ad_group_id)

    # Create a list of ad group criteria operations.
    operations: List[AdGroupCriterionOperation] = []

    # Construct the listing group tree "root" node.
    # Subdivision node: (Root node)
    ad_group_criterion_root_operation: AdGroupCriterionOperation = (
        create_listing_group_subdivision(client, customer_id, ad_group_id)
    )

    # Get the resource name that will be used for the root node.
    # This resource has not been created yet and will include the temporary
    # ID as part of the criterion ID.
    ad_group_criterion_root_resource_name: str = (
        ad_group_criterion_root_operation.create.resource_name
    )
    operations.append(ad_group_criterion_root_operation)

    # Construct the listing group unit nodes for NEW, USED, and other.
    product_condition_enum: ProductConditionEnum = (
        client.enums.ProductConditionEnum
    )
    condition_dimension_info: ListingDimensionInfo = client.get_type(
        "ListingDimensionInfo"
    )

    # Biddable Unit node: (Condition NEW node)
    # * Product Condition: NEW
    # * CPC bid: $0.20
    condition_dimension_info.product_condition.condition = (
        product_condition_enum.NEW
    )
    operations.append(
        create_listing_group_unit_biddable(
            client,
            customer_id,
            ad_group_id,
            ad_group_criterion_root_resource_name,
            condition_dimension_info,
            200_000,
        )
    )

    # Biddable Unit node: (Condition USED node)
    # * Product Condition: USED
    # * CPC bid: $0.10
    condition_dimension_info.product_condition.condition = (
        product_condition_enum.USED
    )
    operations.append(
        create_listing_group_unit_biddable(
            client,
            customer_id,
            ad_group_id,
            ad_group_criterion_root_resource_name,
            condition_dimension_info,
            100_000,
        )
    )

    # Sub-division node: (Condition "other" node)
    # * Product Condition: (not specified)
    # Note that all sibling nodes must have the same dimension type, even if
    # they don't contain a bid.
    client.copy_from(
        condition_dimension_info.product_condition,
        client.get_type("ProductConditionInfo"),
    )
    ad_group_criterion_other_operation: AdGroupCriterionOperation = (
        create_listing_group_subdivision(
            client,
            customer_id,
            ad_group_id,
            ad_group_criterion_root_resource_name,
            condition_dimension_info,
        )
    )
    # Get the resource name that will be used for the condition other node.
    # This resource has not been created yet and will include the temporary
    # ID as part of the criterion ID.
    ad_group_criterion_other_resource_name: str = (
        ad_group_criterion_other_operation.create.resource_name
    )
    operations.append(ad_group_criterion_other_operation)

    # Build the listing group nodes for CoolBrand, CheapBrand, and other.
    brand_dimension_info: ListingDimensionInfo = client.get_type(
        "ListingDimensionInfo"
    )

    # Biddable Unit node: (Brand CoolBrand node)
    # * Brand: CoolBrand
    # * CPC bid: $0.90
    brand_dimension_info.product_brand.value = "CoolBrand"
    operations.append(
        create_listing_group_unit_biddable(
            client,
            customer_id,
            ad_group_id,
            ad_group_criterion_other_resource_name,
            brand_dimension_info,
            900_000,
        )
    )

    # Biddable Unit node: (Brand CheapBrand node)
    # * Brand: CheapBrand
    # * CPC bid: $0.01
    brand_dimension_info.product_brand.value = "CheapBrand"
    operations.append(
        create_listing_group_unit_biddable(
            client,
            customer_id,
            ad_group_id,
            ad_group_criterion_other_resource_name,
            brand_dimension_info,
            10_000,
        )
    )

    # Biddable Unit node: (Brand other node)
    # * CPC bid: $0.05
    client.copy_from(
        brand_dimension_info.product_brand,
        client.get_type("ProductBrandInfo"),
    )
    operations.append(
        create_listing_group_unit_biddable(
            client,
            customer_id,
            ad_group_id,
            ad_group_criterion_other_resource_name,
            brand_dimension_info,
            50_000,
        )
    )

    # Add the ad group criteria.
    mutate_ad_group_criteria_response: MutateGoogleAdsResponse = (
        ad_group_criterion_service.mutate_ad_group_criteria(
            customer_id=customer_id, operations=operations
        )
    )

    # Print the results of the successful mutates.
    print(
        "Added ad group criteria for the listing group tree with the "
        "following resource names:"
    )
    for result in mutate_ad_group_criteria_response.results:
        print(f"\t{result.resource_name}")

    print(f"{len(mutate_ad_group_criteria_response.results)} criteria added.")
    # [END add_shopping_product_listing_group_tree]


def remove_listing_group_tree(
    client: GoogleAdsClient,
    customer_id: str,
    ad_group_id: str,
) -> None:
    """Removes ad group criteria for an ad group's existing listing group tree.

    Args:
        client: An initialized Google Ads client.
        customer_id: The Google Ads customer ID.
        ad_group_id: The ad group ID from which to remove the listing group
            tree.
    """
    # Get the GoogleAdsService client.
    googleads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )

    print("Removing existing listing group tree...")
    # Create a search Google Ads request that will retrieve all listing groups
    # where the parent ad group criterion is NULL (and hence the root node in
    # the tree) for a given ad group id.
    # Note: ad_group_id is used as an int in the query.
    query: str = f"""
        SELECT ad_group_criterion.resource_name
        FROM ad_group_criterion
        WHERE
          ad_group_criterion.type = LISTING_GROUP
          AND ad_group_criterion.listing_group.parent_ad_group_criterion IS NULL
          AND ad_group.id = {ad_group_id}"""

    results: SearchGoogleAdsResponse = googleads_service.search(
        customer_id=customer_id, query=query
    )
    ad_group_criterion_operations: List[AdGroupCriterionOperation] = []

    # Iterate over all rows to find the ad group criteria to remove.
    for row in results:
        criterion: AdGroupCriterion = row.ad_group_criterion
        print(
            "Found an ad group criterion with resource name: "
            f"'{criterion.resource_name}'."
        )
        ad_group_criterion_operation: AdGroupCriterionOperation = (
            client.get_type("AdGroupCriterionOperation")
        )
        ad_group_criterion_operation.remove = criterion.resource_name
        ad_group_criterion_operations.append(ad_group_criterion_operation)

    if ad_group_criterion_operations:
        # Remove the ad group criteria that define the listing group tree.
        ad_group_criterion_service: AdGroupCriterionServiceClient = (
            client.get_service("AdGroupCriterionService")
        )
        response: MutateGoogleAdsResponse = (
            ad_group_criterion_service.mutate_ad_group_criteria(
                customer_id=customer_id,
                operations=ad_group_criterion_operations,
            )
        )
        print(f"Removed {len(response.results)} ad group criteria.")


def create_listing_group_subdivision(
    client: GoogleAdsClient,
    customer_id: str,
    ad_group_id: str,
    parent_ad_group_criterion_resource_name: Optional[str] = None,
    listing_dimension_info: Optional[ListingDimensionInfo] = None,
) -> AdGroupCriterionOperation:
    """Creates a new criterion containing a subdivision listing group node.

    If the parent ad group criterion resource name or listing dimension info are
    not specified, this method creates a root node.

    Args:
        client: An initialized Google Ads client.
        customer_id: The Google Ads customer ID.
        ad_group_id: The ad group ID to which the node will be added.
        parent_ad_group_criterion_resource_name: The string resource name of the
            parent node to which this listing will be attached.
        listing_dimension_info: A ListingDimensionInfo object containing details
            for this listing.

    Returns:
        An AdGroupCriterionOperation containing a populated ad group criterion.
    """
    # Create an ad group criterion operation and populate the criterion.
    operation: AdGroupCriterionOperation = client.get_type(
        "AdGroupCriterionOperation"
    )
    ad_group_criterion: AdGroupCriterion = operation.create
    ad_group_criterion_service: AdGroupCriterionServiceClient = (
        client.get_service("AdGroupCriterionService")
    )
    # The resource name the criterion will be created with. This will define
    # the ID for the ad group criterion.
    ad_group_criterion.resource_name = (
        ad_group_criterion_service.ad_group_criterion_path(
            customer_id, ad_group_id, next_id()
        )
    )
    ad_group_criterion.status = client.enums.AdGroupCriterionStatusEnum.ENABLED

    listing_group_info: ListingGroupInfo = ad_group_criterion.listing_group
    # Set the type as a SUBDIVISION, which will allow the node to be the
    # parent of another sub-tree.
    listing_group_info.type_ = client.enums.ListingGroupTypeEnum.SUBDIVISION
    # If parent_ad_group_criterion_resource_name and listing_dimension_info
    # are not null, create a non-root division by setting its parent and case
    # value.
    if (
        parent_ad_group_criterion_resource_name
        and listing_dimension_info is not None
    ):
        # Set the ad group criterion resource name for the parent listing group.
        # This can include a temporary ID if the parent criterion is not yet
        # created.
        listing_group_info.parent_ad_group_criterion = (
            parent_ad_group_criterion_resource_name
        )

        # Case values contain the listing dimension used for the node.
        client.copy_from(listing_group_info.case_value, listing_dimension_info)

    return operation


def create_listing_group_unit_biddable(
    client: GoogleAdsClient,
    customer_id: str,
    ad_group_id: str,
    parent_ad_group_criterion_resource_name: str,
    listing_dimension_info: ListingDimensionInfo,
    cpc_bid_micros: Optional[int] = None,
) -> AdGroupCriterionOperation:
    """Creates a new criterion containing a biddable unit listing group node.

    Args:
        client: An initialized Google Ads client.
        customer_id: The Google Ads customer ID.
        ad_group_id: The ad group ID to which the node will be added.
        parent_ad_group_criterion_resource_name: The string resource name of the
            parent node to which this listing will be attached.
        listing_dimension_info: A ListingDimensionInfo object containing details
            for this listing.
        cpc_bid_micros: The cost-per-click bid for this listing in micros.

    Returns:
        An AdGroupCriterionOperation with a populated create field.
    """
    # Note: There are two approaches for creating new unit nodes:
    # (1) Set the ad group resource name on the criterion (no temporary ID
    # required).
    # (2) Use a temporary ID to construct the criterion resource name and set
    # it to the 'resourceName' attribute.
    # In both cases you must set the parent ad group criterion's resource name
    # on the listing group for non-root nodes.
    # This example demonstrates method (1).
    operation: AdGroupCriterionOperation = client.get_type(
        "AdGroupCriterionOperation"
    )

    criterion: AdGroupCriterion = operation.create
    ad_group_service: AdGroupServiceClient = client.get_service(
        "AdGroupService"
    )
    criterion.ad_group = ad_group_service.ad_group_path(
        customer_id, ad_group_id
    )
    criterion.status = client.enums.AdGroupCriterionStatusEnum.ENABLED
    # Set the bid for this listing group unit.
    # This will be used as the CPC bid for items that are included in this
    # listing group.
    if cpc_bid_micros:
        criterion.cpc_bid_micros = cpc_bid_micros

    listing_group: ListingGroupInfo = criterion.listing_group
    # Set the type as a UNIT, which will allow the group to be biddable.
    listing_group.type_ = client.enums.ListingGroupTypeEnum.UNIT
    # Set the ad group criterion resource name for the parent listing group.
    # This can have a temporary ID if the parent criterion is not yet created.
    listing_group.parent_ad_group_criterion = (
        parent_ad_group_criterion_resource_name
    )
    # Case values contain the listing dimension used for the node.
    if listing_dimension_info is not None:
        client.copy_from(listing_group.case_value, listing_dimension_info)

    return operation


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Add shopping product listing group tree to a shopping ad "
        "group."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-a",
        "--ad_group_id",
        type=str,
        required=True,
        help="The ID of the ad group that will receive the listing group tree.",
    )
    parser.add_argument(
        "-r",
        "--replace_existing_tree",
        action="store_true",
        required=False,
        default=False,
        help="Optional, whether to replace the existing listing group tree on "
        "the ad group if one already exists. Defaults to false.",
    )

    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(
            googleads_client,
            args.customer_id,
            args.ad_group_id,
            args.replace_existing_tree,
        )
    except GoogleAdsException as ex:
        print(
            f"Request with ID '{ex.request_id}' failed with status "
            f"'{ex.error.code().name}' and includes the following errors:"
        )
        for error in ex.failure.errors:
            print(f"\tError with message '{error.message}'.")
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "SimulationModificationMethodEnum",
    },
)


class SimulationModificationMethodEnum(proto.Message):
    r"""Container for enum describing the method by which a
    simulation modifies a field.

    """

    class SimulationModificationMethod(proto.Enum):
        r"""Enum describing the method by which a simulation modifies a
        field.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            UNIFORM (2):
                The values in a simulation were applied to
                all children of a given resource uniformly.
                Overrides on child resources were not respected.
            DEFAULT (3):
                The values in a simulation were applied to
                the given resource. Overrides on child resources
                were respected, and traffic estimates do not
                include these resources.
            SCALING (4):
                The values in a simulation were all scaled by
                the same factor. For example, in a simulated
                TargetCpa campaign, the campaign target and all
                ad group targets were scaled by a factor of X.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        UNIFORM = 2
        DEFAULT = 3
        SCALING = 4


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.resources.types import campaign_goal_config
from google.protobuf import field_mask_pb2  # type: ignore
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.services",
    marshal="google.ads.googleads.v22",
    manifest={
        "MutateCampaignGoalConfigsRequest",
        "CampaignGoalConfigOperation",
        "MutateCampaignGoalConfigsResponse",
        "MutateCampaignGoalConfigResult",
    },
)


class MutateCampaignGoalConfigsRequest(proto.Message):
    r"""Request message for
    [CampaignGoalConfigService.MutateCampaignGoalConfigs][google.ads.googleads.v22.services.CampaignGoalConfigService.MutateCampaignGoalConfigs].

    Attributes:
        customer_id (str):
            Required. The ID of the customer whose
            campaign goal configs are being modified.
        operations (MutableSequence[google.ads.googleads.v22.services.types.CampaignGoalConfigOperation]):
            Required. The list of operations to perform
            on the campaign goal configs.
        partial_failure (bool):
            Optional. If true, successful operations will
            be carried out and invalid operations will
            return errors. If false, all operations will be
            carried out in one transaction if and only if
            they are all valid. Default is false.
        validate_only (bool):
            Optional. If true, the request is validated
            but not executed. Only errors are returned, not
            results.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operations: MutableSequence["CampaignGoalConfigOperation"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="CampaignGoalConfigOperation",
        )
    )
    partial_failure: bool = proto.Field(
        proto.BOOL,
        number=3,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=4,
    )


class CampaignGoalConfigOperation(proto.Message):
    r"""A single mutate operation on the campaign goal config.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            FieldMask that determines which fields are
            modified in an update.
        create (google.ads.googleads.v22.resources.types.CampaignGoalConfig):
            Create a new campaign goal config.

            This field is a member of `oneof`_ ``operation``.
        update (google.ads.googleads.v22.resources.types.CampaignGoalConfig):
            Update an existing campaign goal config.

            This field is a member of `oneof`_ ``operation``.
        remove (str):
            Remove an existing campaign goal config.

            This field is a member of `oneof`_ ``operation``.
    """

    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=4,
        message=field_mask_pb2.FieldMask,
    )
    create: campaign_goal_config.CampaignGoalConfig = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="operation",
        message=campaign_goal_config.CampaignGoalConfig,
    )
    update: campaign_goal_config.CampaignGoalConfig = proto.Field(
        proto.MESSAGE,
        number=3,
        oneof="operation",
        message=campaign_goal_config.CampaignGoalConfig,
    )
    remove: str = proto.Field(
        proto.STRING,
        number=2,
        oneof="operation",
    )


class MutateCampaignGoalConfigsResponse(proto.Message):
    r"""Response message for a campaign goal config mutate.

    Attributes:
        partial_failure_error (google.rpc.status_pb2.Status):
            Errors that pertain to operation failures in
            the partial failure mode.
        results (MutableSequence[google.ads.googleads.v22.services.types.MutateCampaignGoalConfigResult]):
            All results for the mutate.
    """

    partial_failure_error: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=1,
        message=status_pb2.Status,
    )
    results: MutableSequence["MutateCampaignGoalConfigResult"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="MutateCampaignGoalConfigResult",
        )
    )


class MutateCampaignGoalConfigResult(proto.Message):
    r"""The result for the campaign goal config mutate.

    Attributes:
        resource_name (str):
            Returned for successful operations.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "AdGroupAdPrimaryStatusReasonEnum",
    },
)


class AdGroupAdPrimaryStatusReasonEnum(proto.Message):
    r"""AdGroupAd Primary Status Reason. Provides insight into why an
    ad group ad is not serving or not serving optimally. These
    reasons are aggregated to determine an overall
    AdGroupAdPrimaryStatus.

    """

    class AdGroupAdPrimaryStatusReason(proto.Enum):
        r"""Possible ad group ad status reasons.

        Values:
            UNSPECIFIED (0):
                No value has been specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            CAMPAIGN_REMOVED (2):
                The user-specified campaign status is
                removed. Contributes to
                AdGroupAdPrimaryStatus.REMOVED.
            CAMPAIGN_PAUSED (3):
                The user-specified campaign status is paused.
                Contributes to AdGroupAdPrimaryStatus.PAUSED.
            CAMPAIGN_PENDING (4):
                The user-specified time for this campaign to
                start is in the future. Contributes to
                AdGroupAdPrimaryStatus.PENDING.
            CAMPAIGN_ENDED (5):
                The user-specified time for this campaign to
                end has passed. Contributes to
                AdGroupAdPrimaryStatus.ENDED.
            AD_GROUP_PAUSED (6):
                The user-specified ad group status is paused.
                Contributes to AdGroupAdPrimaryStatus.PAUSED.
            AD_GROUP_REMOVED (7):
                The user-specified ad group status is
                removed. Contributes to
                AdGroupAdPrimaryStatus.REMOVED.
            AD_GROUP_AD_PAUSED (8):
                The user-specified ad status is paused.
                Contributes to AdGroupAdPrimaryStatus.PAUSED.
            AD_GROUP_AD_REMOVED (9):
                The user-specified ad status is removed.
                Contributes to AdGroupAdPrimaryStatus.REMOVED.
            AD_GROUP_AD_DISAPPROVED (10):
                The ad is disapproved. Contributes to
                AdGroupAdPrimaryStatus.NOT_ELIGIBLE.
            AD_GROUP_AD_UNDER_REVIEW (11):
                The ad is under review. Contributes to
                AdGroupAdPrimaryStatus.PENDING.
            AD_GROUP_AD_POOR_QUALITY (12):
                The ad is poor quality. This is determined by
                the serving stack that served the Ad.
                Contributes to AdGroupAdPrimaryStatus.LIMITED.
            AD_GROUP_AD_NO_ADS (13):
                No eligible ads instances could be generated.
                Contributes to AdGroupAdPrimaryStatus.PENDING.
            AD_GROUP_AD_APPROVED_LABELED (14):
                The ad is internally labeled with a limiting
                label. Contributes to
                AdGroupAdPrimaryStatus.LIMITED.
            AD_GROUP_AD_AREA_OF_INTEREST_ONLY (15):
                The ad is only serving in the user-specified
                area of interest. Contributes to
                AdGroupAdPrimaryStatus.LIMITED.
            AD_GROUP_AD_UNDER_APPEAL (16):
                The ad is part of an ongoing appeal. This
                reason does not impact AdGroupAdPrimaryStatus.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CAMPAIGN_REMOVED = 2
        CAMPAIGN_PAUSED = 3
        CAMPAIGN_PENDING = 4
        CAMPAIGN_ENDED = 5
        AD_GROUP_PAUSED = 6
        AD_GROUP_REMOVED = 7
        AD_GROUP_AD_PAUSED = 8
        AD_GROUP_AD_REMOVED = 9
        AD_GROUP_AD_DISAPPROVED = 10
        AD_GROUP_AD_UNDER_REVIEW = 11
        AD_GROUP_AD_POOR_QUALITY = 12
        AD_GROUP_AD_NO_ADS = 13
        AD_GROUP_AD_APPROVED_LABELED = 14
        AD_GROUP_AD_AREA_OF_INTEREST_ONLY = 15
        AD_GROUP_AD_UNDER_APPEAL = 16


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v21.enums.types import (
    third_party_brand_lift_integration_partner,
)
from google.ads.googleads.v21.enums.types import (
    third_party_brand_safety_integration_partner,
)
from google.ads.googleads.v21.enums.types import (
    third_party_reach_integration_partner,
)
from google.ads.googleads.v21.enums.types import (
    third_party_viewability_integration_partner,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.common",
    marshal="google.ads.googleads.v21",
    manifest={
        "CustomerThirdPartyIntegrationPartners",
        "CustomerThirdPartyViewabilityIntegrationPartner",
        "CustomerThirdPartyBrandSafetyIntegrationPartner",
        "CustomerThirdPartyBrandLiftIntegrationPartner",
        "CustomerThirdPartyReachIntegrationPartner",
        "CampaignThirdPartyIntegrationPartners",
        "CampaignThirdPartyViewabilityIntegrationPartner",
        "CampaignThirdPartyBrandSafetyIntegrationPartner",
        "CampaignThirdPartyBrandLiftIntegrationPartner",
        "CampaignThirdPartyReachIntegrationPartner",
        "ThirdPartyIntegrationPartnerData",
    },
)


class CustomerThirdPartyIntegrationPartners(proto.Message):
    r"""Container for Customer level third party integration
    partners. Next Id = 5

    Attributes:
        viewability_integration_partners (MutableSequence[google.ads.googleads.v21.common.types.CustomerThirdPartyViewabilityIntegrationPartner]):
            Allowed third party integration partners for
            YouTube viewability verification.
        brand_lift_integration_partners (MutableSequence[google.ads.googleads.v21.common.types.CustomerThirdPartyBrandLiftIntegrationPartner]):
            Allowed third party integration partners for
            Brand Lift verification.
        brand_safety_integration_partners (MutableSequence[google.ads.googleads.v21.common.types.CustomerThirdPartyBrandSafetyIntegrationPartner]):
            Allowed third party integration partners for
            brand safety verification.
        reach_integration_partners (MutableSequence[google.ads.googleads.v21.common.types.CustomerThirdPartyReachIntegrationPartner]):
            Allowed third party integration partners for
            reach verification.
    """

    viewability_integration_partners: MutableSequence[
        "CustomerThirdPartyViewabilityIntegrationPartner"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="CustomerThirdPartyViewabilityIntegrationPartner",
    )
    brand_lift_integration_partners: MutableSequence[
        "CustomerThirdPartyBrandLiftIntegrationPartner"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="CustomerThirdPartyBrandLiftIntegrationPartner",
    )
    brand_safety_integration_partners: MutableSequence[
        "CustomerThirdPartyBrandSafetyIntegrationPartner"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message="CustomerThirdPartyBrandSafetyIntegrationPartner",
    )
    reach_integration_partners: MutableSequence[
        "CustomerThirdPartyReachIntegrationPartner"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=4,
        message="CustomerThirdPartyReachIntegrationPartner",
    )


class CustomerThirdPartyViewabilityIntegrationPartner(proto.Message):
    r"""Container for third party viewability integration data for
    Customer. Next Id = 3

    Attributes:
        viewability_integration_partner (google.ads.googleads.v21.enums.types.ThirdPartyViewabilityIntegrationPartnerEnum.ThirdPartyViewabilityIntegrationPartner):
            Allowed third party integration partners for
            YouTube viewability verification.
        allow_share_cost (bool):
            If true, cost data can be shared with this
            vendor.
    """

    viewability_integration_partner: (
        third_party_viewability_integration_partner.ThirdPartyViewabilityIntegrationPartnerEnum.ThirdPartyViewabilityIntegrationPartner
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=third_party_viewability_integration_partner.ThirdPartyViewabilityIntegrationPartnerEnum.ThirdPartyViewabilityIntegrationPartner,
    )
    allow_share_cost: bool = proto.Field(
        proto.BOOL,
        number=2,
    )


class CustomerThirdPartyBrandSafetyIntegrationPartner(proto.Message):
    r"""Container for third party brand safety integration data for
    Customer. Next Id = 2

    Attributes:
        brand_safety_integration_partner (google.ads.googleads.v21.enums.types.ThirdPartyBrandSafetyIntegrationPartnerEnum.ThirdPartyBrandSafetyIntegrationPartner):
            Allowed third party integration partners for
            brand safety verification.
    """

    brand_safety_integration_partner: (
        third_party_brand_safety_integration_partner.ThirdPartyBrandSafetyIntegrationPartnerEnum.ThirdPartyBrandSafetyIntegrationPartner
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=third_party_brand_safety_integration_partner.ThirdPartyBrandSafetyIntegrationPartnerEnum.ThirdPartyBrandSafetyIntegrationPartner,
    )


class CustomerThirdPartyBrandLiftIntegrationPartner(proto.Message):
    r"""Container for third party Brand Lift integration data for
    Customer. Next Id = 3

    Attributes:
        brand_lift_integration_partner (google.ads.googleads.v21.enums.types.ThirdPartyBrandLiftIntegrationPartnerEnum.ThirdPartyBrandLiftIntegrationPartner):
            Allowed Third Party integration partners for
            Brand Lift verification.
        allow_share_cost (bool):
            If true, cost data can be shared with this
            vendor.
    """

    brand_lift_integration_partner: (
        third_party_brand_lift_integration_partner.ThirdPartyBrandLiftIntegrationPartnerEnum.ThirdPartyBrandLiftIntegrationPartner
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=third_party_brand_lift_integration_partner.ThirdPartyBrandLiftIntegrationPartnerEnum.ThirdPartyBrandLiftIntegrationPartner,
    )
    allow_share_cost: bool = proto.Field(
        proto.BOOL,
        number=2,
    )


class CustomerThirdPartyReachIntegrationPartner(proto.Message):
    r"""Container for third party reach integration data for
    Customer. Next Id = 3

    Attributes:
        reach_integration_partner (google.ads.googleads.v21.enums.types.ThirdPartyReachIntegrationPartnerEnum.ThirdPartyReachIntegrationPartner):
            Allowed Third Party integration partners for
            reach verification.
        allow_share_cost (bool):
            If true, cost data can be shared with this
            vendor.
    """

    reach_integration_partner: (
        third_party_reach_integration_partner.ThirdPartyReachIntegrationPartnerEnum.ThirdPartyReachIntegrationPartner
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=third_party_reach_integration_partner.ThirdPartyReachIntegrationPartnerEnum.ThirdPartyReachIntegrationPartner,
    )
    allow_share_cost: bool = proto.Field(
        proto.BOOL,
        number=2,
    )


class CampaignThirdPartyIntegrationPartners(proto.Message):
    r"""Container for Campaign level third party integration
    partners. Next Id = 5

    Attributes:
        viewability_integration_partners (MutableSequence[google.ads.googleads.v21.common.types.CampaignThirdPartyViewabilityIntegrationPartner]):
            Third party integration partners for YouTube
            viewability verification for this Campaign.
        brand_lift_integration_partners (MutableSequence[google.ads.googleads.v21.common.types.CampaignThirdPartyBrandLiftIntegrationPartner]):
            Third party integration partners for Brand
            Lift verification for this Campaign.
        brand_safety_integration_partners (MutableSequence[google.ads.googleads.v21.common.types.CampaignThirdPartyBrandSafetyIntegrationPartner]):
            Third party integration partners for brand
            safety verification for this Campaign.
        reach_integration_partners (MutableSequence[google.ads.googleads.v21.common.types.CampaignThirdPartyReachIntegrationPartner]):
            Third party integration partners for reach
            verification for this Campaign.
    """

    viewability_integration_partners: MutableSequence[
        "CampaignThirdPartyViewabilityIntegrationPartner"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="CampaignThirdPartyViewabilityIntegrationPartner",
    )
    brand_lift_integration_partners: MutableSequence[
        "CampaignThirdPartyBrandLiftIntegrationPartner"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="CampaignThirdPartyBrandLiftIntegrationPartner",
    )
    brand_safety_integration_partners: MutableSequence[
        "CampaignThirdPartyBrandSafetyIntegrationPartner"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message="CampaignThirdPartyBrandSafetyIntegrationPartner",
    )
    reach_integration_partners: MutableSequence[
        "CampaignThirdPartyReachIntegrationPartner"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=4,
        message="CampaignThirdPartyReachIntegrationPartner",
    )


class CampaignThirdPartyViewabilityIntegrationPartner(proto.Message):
    r"""Container for third party viewability integration data for
    Campaign. Next Id = 4

    Attributes:
        viewability_integration_partner (google.ads.googleads.v21.enums.types.ThirdPartyViewabilityIntegrationPartnerEnum.ThirdPartyViewabilityIntegrationPartner):
            Allowed third party integration partners for
            YouTube viewability verification.
        viewability_integration_partner_data (google.ads.googleads.v21.common.types.ThirdPartyIntegrationPartnerData):
            Third party partner data for YouTube
            viewability verification. This is optional
            metadata for partners to join or attach data to
            Ads campaigns.
        share_cost (bool):
            If true, then cost data will be shared with
            this vendor.
    """

    viewability_integration_partner: (
        third_party_viewability_integration_partner.ThirdPartyViewabilityIntegrationPartnerEnum.ThirdPartyViewabilityIntegrationPartner
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=third_party_viewability_integration_partner.ThirdPartyViewabilityIntegrationPartnerEnum.ThirdPartyViewabilityIntegrationPartner,
    )
    viewability_integration_partner_data: "ThirdPartyIntegrationPartnerData" = (
        proto.Field(
            proto.MESSAGE,
            number=2,
            message="ThirdPartyIntegrationPartnerData",
        )
    )
    share_cost: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class CampaignThirdPartyBrandSafetyIntegrationPartner(proto.Message):
    r"""Container for third party brand safety integration data for
    Campaign. Next Id = 3

    Attributes:
        brand_safety_integration_partner (google.ads.googleads.v21.enums.types.ThirdPartyBrandSafetyIntegrationPartnerEnum.ThirdPartyBrandSafetyIntegrationPartner):
            Allowed third party integration partners for
            brand safety verification.
        brand_safety_integration_partner_data (google.ads.googleads.v21.common.types.ThirdPartyIntegrationPartnerData):
            Third party partner data for YouTube brand
            safety verification. This is optional metadata
            for partners to join or attach data to Ads
            campaigns.
    """

    brand_safety_integration_partner: (
        third_party_brand_safety_integration_partner.ThirdPartyBrandSafetyIntegrationPartnerEnum.ThirdPartyBrandSafetyIntegrationPartner
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=third_party_brand_safety_integration_partner.ThirdPartyBrandSafetyIntegrationPartnerEnum.ThirdPartyBrandSafetyIntegrationPartner,
    )
    brand_safety_integration_partner_data: (
        "ThirdPartyIntegrationPartnerData"
    ) = proto.Field(
        proto.MESSAGE,
        number=2,
        message="ThirdPartyIntegrationPartnerData",
    )


class CampaignThirdPartyBrandLiftIntegrationPartner(proto.Message):
    r"""Container for third party Brand Lift integration data for
    Campaign. Next Id = 4

    Attributes:
        brand_lift_integration_partner (google.ads.googleads.v21.enums.types.ThirdPartyBrandLiftIntegrationPartnerEnum.ThirdPartyBrandLiftIntegrationPartner):
            Allowed third party integration partners for
            Brand Lift verification.
        brand_lift_integration_partner_data (google.ads.googleads.v21.common.types.ThirdPartyIntegrationPartnerData):
            Third party partner data for YouTube Brand
            Lift verification. This is optional metadata for
            partners to join or attach data to Ads
            campaigns.
        share_cost (bool):
            If true, then cost data will be shared with
            this vendor.
    """

    brand_lift_integration_partner: (
        third_party_brand_lift_integration_partner.ThirdPartyBrandLiftIntegrationPartnerEnum.ThirdPartyBrandLiftIntegrationPartner
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=third_party_brand_lift_integration_partner.ThirdPartyBrandLiftIntegrationPartnerEnum.ThirdPartyBrandLiftIntegrationPartner,
    )
    brand_lift_integration_partner_data: "ThirdPartyIntegrationPartnerData" = (
        proto.Field(
            proto.MESSAGE,
            number=2,
            message="ThirdPartyIntegrationPartnerData",
        )
    )
    share_cost: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class CampaignThirdPartyReachIntegrationPartner(proto.Message):
    r"""Container for third party reach integration data for
    Campaign. Next Id = 4

    Attributes:
        reach_integration_partner (google.ads.googleads.v21.enums.types.ThirdPartyReachIntegrationPartnerEnum.ThirdPartyReachIntegrationPartner):
            Allowed third party integration partners for
            reach verification.
        reach_integration_partner_data (google.ads.googleads.v21.common.types.ThirdPartyIntegrationPartnerData):
            Third party partner data for YouTube Reach
            verification. This is optional metadata for
            partners to join or attach data to Ads
            campaigns.
        share_cost (bool):
            If true, then cost data will be shared with
            this vendor.
    """

    reach_integration_partner: (
        third_party_reach_integration_partner.ThirdPartyReachIntegrationPartnerEnum.ThirdPartyReachIntegrationPartner
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=third_party_reach_integration_partner.ThirdPartyReachIntegrationPartnerEnum.ThirdPartyReachIntegrationPartner,
    )
    reach_integration_partner_data: "ThirdPartyIntegrationPartnerData" = (
        proto.Field(
            proto.MESSAGE,
            number=2,
            message="ThirdPartyIntegrationPartnerData",
        )
    )
    share_cost: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class ThirdPartyIntegrationPartnerData(proto.Message):
    r"""Contains third party measurement partner related data for
    video campaigns. Next Id = 3

    Attributes:
        client_id (str):
            The client ID that allows the measurement
            partner to join multiple campaigns for a
            particular advertiser.
        third_party_placement_id (str):
            The third party placement ID that maps the
            measurement partner data with a campaign (or a
            group of related campaigns) specific data.
    """

    client_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    third_party_placement_id: str = proto.Field(
        proto.STRING,
        number=2,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2018 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example adds an ad group.

To get ad groups, run get_ad_groups.py.
"""


import argparse
import sys
from typing import List
import uuid

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.services.services.ad_group_service import (
    AdGroupServiceClient,
)
from google.ads.googleads.v22.services.types.ad_group_service import (
    AdGroupOperation,
    MutateAdGroupsResponse,
)
from google.ads.googleads.v22.services.services.campaign_service import (
    CampaignServiceClient,
)
from google.ads.googleads.v22.resources.types.ad_group import AdGroup


def main(client: GoogleAdsClient, customer_id: str, campaign_id: str) -> None:
    ad_group_service: AdGroupServiceClient = client.get_service(
        "AdGroupService"
    )
    campaign_service: CampaignServiceClient = client.get_service(
        "CampaignService"
    )

    # Create ad group.
    ad_group_operation: AdGroupOperation = client.get_type("AdGroupOperation")
    ad_group: AdGroup = ad_group_operation.create
    ad_group.name = f"Earth to Mars cruises {uuid.uuid4()}"
    ad_group.status = client.enums.AdGroupStatusEnum.ENABLED
    ad_group.campaign = campaign_service.campaign_path(customer_id, campaign_id)
    ad_group.type_ = client.enums.AdGroupTypeEnum.SEARCH_STANDARD
    ad_group.cpc_bid_micros = 10000000

    operations: List[AdGroupOperation] = [ad_group_operation]

    # Add the ad group.
    ad_group_response: MutateAdGroupsResponse = (
        ad_group_service.mutate_ad_groups(
            customer_id=customer_id,
            operations=operations,
        )
    )
    print(f"Created ad group {ad_group_response.results[0].resource_name}.")


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Adds an ad group for specified customer and campaign id."
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-i", "--campaign_id", type=str, required=True, help="The campaign ID."
    )
    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(googleads_client, args.customer_id, args.campaign_id)
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'\tError with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "DisplayUploadProductTypeEnum",
    },
)


class DisplayUploadProductTypeEnum(proto.Message):
    r"""Container for display upload product types. Product types
    that have the word "DYNAMIC" in them must be associated with a
    campaign that has a dynamic remarketing feed. See
    https://support.google.com/google-ads/answer/6053288 for more
    info about dynamic remarketing. Other product types are regarded
    as "static" and do not have this requirement.

    """

    class DisplayUploadProductType(proto.Enum):
        r"""Enumerates display upload product types.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                The value is unknown in this version.
            HTML5_UPLOAD_AD (2):
                HTML5 upload ad. This product type requires the
                upload_media_bundle field in DisplayUploadAdInfo to be set.
            DYNAMIC_HTML5_EDUCATION_AD (3):
                Dynamic HTML5 education ad. This product type requires the
                upload_media_bundle field in DisplayUploadAdInfo to be set.
                Can only be used in an education campaign.
            DYNAMIC_HTML5_FLIGHT_AD (4):
                Dynamic HTML5 flight ad. This product type requires the
                upload_media_bundle field in DisplayUploadAdInfo to be set.
                Can only be used in a flight campaign.
            DYNAMIC_HTML5_HOTEL_RENTAL_AD (5):
                Dynamic HTML5 hotel and rental ad. This product type
                requires the upload_media_bundle field in
                DisplayUploadAdInfo to be set. Can only be used in a hotel
                campaign.
            DYNAMIC_HTML5_JOB_AD (6):
                Dynamic HTML5 job ad. This product type requires the
                upload_media_bundle field in DisplayUploadAdInfo to be set.
                Can only be used in a job campaign.
            DYNAMIC_HTML5_LOCAL_AD (7):
                Dynamic HTML5 local ad. This product type requires the
                upload_media_bundle field in DisplayUploadAdInfo to be set.
                Can only be used in a local campaign.
            DYNAMIC_HTML5_REAL_ESTATE_AD (8):
                Dynamic HTML5 real estate ad. This product type requires the
                upload_media_bundle field in DisplayUploadAdInfo to be set.
                Can only be used in a real estate campaign.
            DYNAMIC_HTML5_CUSTOM_AD (9):
                Dynamic HTML5 custom ad. This product type requires the
                upload_media_bundle field in DisplayUploadAdInfo to be set.
                Can only be used in a custom campaign.
            DYNAMIC_HTML5_TRAVEL_AD (10):
                Dynamic HTML5 travel ad. This product type requires the
                upload_media_bundle field in DisplayUploadAdInfo to be set.
                Can only be used in a travel campaign.
            DYNAMIC_HTML5_HOTEL_AD (11):
                Dynamic HTML5 hotel ad. This product type requires the
                upload_media_bundle field in DisplayUploadAdInfo to be set.
                Can only be used in a hotel campaign.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        HTML5_UPLOAD_AD = 2
        DYNAMIC_HTML5_EDUCATION_AD = 3
        DYNAMIC_HTML5_FLIGHT_AD = 4
        DYNAMIC_HTML5_HOTEL_RENTAL_AD = 5
        DYNAMIC_HTML5_JOB_AD = 6
        DYNAMIC_HTML5_LOCAL_AD = 7
        DYNAMIC_HTML5_REAL_ESTATE_AD = 8
        DYNAMIC_HTML5_CUSTOM_AD = 9
        DYNAMIC_HTML5_TRAVEL_AD = 10
        DYNAMIC_HTML5_HOTEL_AD = 11


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2020 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example illustrates the use of custom client timeouts.

Even though this example demonstrates custom client timeouts in the context of
streaming and unary calls separately, the behavior can be applied to any request
method exposed by this library.

For more information about the concepts, see this documentation:
https://grpc.io/docs/what-is-grpc/core-concepts/#rpc-life-cycle
"""


import argparse
from collections.abc import Iterator
import sys
from typing import List

from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.v22.services.services.google_ads_service.client import (
    GoogleAdsServiceClient,
)
from google.ads.googleads.v22.services.types.google_ads_service import (
    GoogleAdsRow,
    SearchGoogleAdsRequest,
    SearchGoogleAdsStreamRequest,
    SearchGoogleAdsStreamResponse,
)
from google.api_core.exceptions import DeadlineExceeded
from google.api_core.retry import Retry


_CLIENT_TIMEOUT_SECONDS = 5 * 60  # 5 minutes.
_QUERY: str = "SELECT campaign.id FROM campaign"


def main(client: GoogleAdsClient, customer_id: str) -> None:
    """Main method, to run this code example as a standalone application."""
    make_server_streaming_call(client, customer_id)
    make_unary_call(client, customer_id)


# [START set_custom_client_timeouts]
def make_server_streaming_call(
    client: GoogleAdsClient, customer_id: str
) -> None:
    """Makes a server streaming call using a custom client timeout.

    Args:
        client: An initialized GoogleAds client.
        customer_id: The str Google Ads customer ID.
    """
    ga_service: GoogleAdsServiceClient = client.get_service("GoogleAdsService")
    campaign_ids: List[str] = []

    try:
        search_request: SearchGoogleAdsStreamRequest = client.get_type(
            "SearchGoogleAdsStreamRequest"
        )
        search_request.customer_id = customer_id
        search_request.query = _QUERY
        stream: Iterator[SearchGoogleAdsStreamResponse] = (
            ga_service.search_stream(
                request=search_request,
                # When making any request, an optional "timeout" parameter can be
                # provided to specify a client-side response deadline in seconds.
                # If not set, then no timeout will be enforced by the client and
                # the channel will remain open until the response is completed or
                # severed, either manually or by the server.
                timeout=_CLIENT_TIMEOUT_SECONDS,
            )
        )

        batch: SearchGoogleAdsStreamResponse
        for batch in stream:
            row: GoogleAdsRow
            for row in batch.results:
                campaign_ids.append(row.campaign.id)

        print("The server streaming call completed before the timeout.")
    except DeadlineExceeded:
        print("The server streaming call did not complete before the timeout.")
        sys.exit(1)
    except GoogleAdsException as ex:
        print(
            f"Request with ID '{ex.request_id}' failed with status "
            f"'{ex.error.code().name}' and includes the following errors:"
        )
        for error in ex.failure.errors:
            print(f"\tError with message '{error.message}'.")
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)

    print(f"Total # of campaign IDs retrieved: {len(campaign_ids)}")
    # [END set_custom_client_timeouts]


# [START set_custom_client_timeouts_1]
def make_unary_call(client: GoogleAdsClient, customer_id: str) -> None:
    """Makes a unary call using a custom client timeout.

    Args:
        client: An initialized GoogleAds client.
        customer_id: The Google Ads customer ID.
    """
    ga_service: GoogleAdsServiceClient = client.get_service("GoogleAdsService")
    campaign_ids: List[str] = []

    try:
        search_request: SearchGoogleAdsRequest = client.get_type(
            "SearchGoogleAdsRequest"
        )
        search_request.customer_id = customer_id
        search_request.query = _QUERY
        results: Iterator[GoogleAdsRow] = ga_service.search(
            request=search_request,
            # When making any request, an optional "retry" parameter can be
            # provided to specify its retry behavior. Complete information about
            # these settings can be found here:
            # https://googleapis.dev/python/google-api-core/latest/retry.html
            retry=Retry(
                # Sets the maximum accumulative timeout of the call; it
                # includes all tries.
                deadline=_CLIENT_TIMEOUT_SECONDS,
                # Sets the timeout that is used for the first try to one tenth
                # of the maximum accumulative timeout of the call.
                # Note: This overrides the default value and can lead to
                # RequestError.RPC_DEADLINE_TOO_SHORT errors when too small. We
                # recommend changing the value only if necessary.
                initial=_CLIENT_TIMEOUT_SECONDS / 10,
                # Sets the maximum timeout that can be used for any given try
                # to one fifth of the maximum accumulative timeout of the call
                # (two times greater than the timeout that is needed for the
                # first try).
                maximum=_CLIENT_TIMEOUT_SECONDS / 5,
            ),
        )

        row: GoogleAdsRow
        for row in results:
            campaign_ids.append(row.campaign.id)

        print("The unary call completed before the timeout.")
    except DeadlineExceeded:
        print("The unary call did not complete before the timeout.")
        sys.exit(1)
    except GoogleAdsException as ex:
        print(
            f"Request with ID '{ex.request_id}' failed with status "
            f"'{ex.error.code().name}' and includes the following errors:"
        )
        for error in ex.failure.errors:
            print(f"\tError with message '{error.message}'.")
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)

    print(f"Total # of campaign IDs retrieved: {len(campaign_ids)}")
    # [END set_custom_client_timeouts_1]


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description=(
            "Demonstrates custom client timeouts in the context of "
            "server streaming and unary calls."
        )
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    args = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client = GoogleAdsClient.load_from_storage(version="v22")

    main(googleads_client, args.customer_id)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.errors",
    marshal="google.ads.googleads.v22",
    manifest={
        "ConversionValueRuleSetErrorEnum",
    },
)


class ConversionValueRuleSetErrorEnum(proto.Message):
    r"""Container for enum describing possible conversion value rule
    set errors.

    """

    class ConversionValueRuleSetError(proto.Enum):
        r"""Enum describing possible conversion value rule set errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            CONFLICTING_VALUE_RULE_CONDITIONS (2):
                Two value rules in this value rule set
                contain conflicting conditions.
            INVALID_VALUE_RULE (3):
                This value rule set includes a value rule
                that cannot be found, has been permanently
                removed or belongs to a different customer.
            DIMENSIONS_UPDATE_ONLY_ALLOW_APPEND (4):
                An error that's thrown when a mutate
                operation is trying to replace/remove some
                existing elements in the dimensions field. In
                other words, ADD op is always fine and UPDATE op
                is fine if it's only appending new elements into
                dimensions list.
            CONDITION_TYPE_NOT_ALLOWED (5):
                An error that's thrown when a mutate is
                adding new value rule(s) into a value rule set
                and the added value rule(s) include conditions
                that are not specified in the dimensions of the
                value rule set.
            DUPLICATE_DIMENSIONS (6):
                The dimensions field contains duplicate
                elements.
            INVALID_CAMPAIGN_ID (7):
                This value rule set is attached to an invalid
                campaign id. Either a campaign with this
                campaign id doesn't exist or it belongs to a
                different customer.
            CANNOT_PAUSE_UNLESS_ALL_VALUE_RULES_ARE_PAUSED (8):
                When a mutate request tries to pause a value
                rule set, the enabled value rules in this set
                must be paused in the same command, or this
                error will be thrown.
            SHOULD_PAUSE_WHEN_ALL_VALUE_RULES_ARE_PAUSED (9):
                When a mutate request tries to pause all the
                value rules in a value rule set, the value rule
                set must be paused, or this error will be
                thrown.
            VALUE_RULES_NOT_SUPPORTED_FOR_CAMPAIGN_TYPE (10):
                This value rule set is attached to a campaign
                that does not support value rules. Currently,
                campaign level value rule sets can only be
                created on Search, or Display campaigns.
            INELIGIBLE_CONVERSION_ACTION_CATEGORIES (11):
                To add a value rule set that applies on Store
                Visits/Store Sales conversion action categories,
                the customer must have valid Store Visits/ Store
                Sales conversion actions.
            DIMENSION_NO_CONDITION_USED_WITH_OTHER_DIMENSIONS (12):
                If NO_CONDITION is used as a dimension of a value rule set,
                it must be the only dimension.
            DIMENSION_NO_CONDITION_NOT_ALLOWED (13):
                Dimension NO_CONDITION can only be used by Store
                Visits/Store Sales value rule set.
            UNSUPPORTED_CONVERSION_ACTION_CATEGORIES (14):
                Value rule sets defined on the specified conversion action
                categories are not supported. The list of conversion action
                categories must be an empty list, only STORE_VISIT, or only
                STORE_SALE.
            DIMENSION_NOT_SUPPORTED_FOR_CAMPAIGN_TYPE (15):
                Dimension ITINERARY can only be used on campaigns with an
                advertising channel type of PERFORMANCE_MAX or HOTEL.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CONFLICTING_VALUE_RULE_CONDITIONS = 2
        INVALID_VALUE_RULE = 3
        DIMENSIONS_UPDATE_ONLY_ALLOW_APPEND = 4
        CONDITION_TYPE_NOT_ALLOWED = 5
        DUPLICATE_DIMENSIONS = 6
        INVALID_CAMPAIGN_ID = 7
        CANNOT_PAUSE_UNLESS_ALL_VALUE_RULES_ARE_PAUSED = 8
        SHOULD_PAUSE_WHEN_ALL_VALUE_RULES_ARE_PAUSED = 9
        VALUE_RULES_NOT_SUPPORTED_FOR_CAMPAIGN_TYPE = 10
        INELIGIBLE_CONVERSION_ACTION_CATEGORIES = 11
        DIMENSION_NO_CONDITION_USED_WITH_OTHER_DIMENSIONS = 12
        DIMENSION_NO_CONDITION_NOT_ALLOWED = 13
        UNSUPPORTED_CONVERSION_ACTION_CATEGORIES = 14
        DIMENSION_NOT_SUPPORTED_FOR_CAMPAIGN_TYPE = 15


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.enums.types import (
    targeting_dimension as gage_targeting_dimension,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.common",
    marshal="google.ads.googleads.v22",
    manifest={
        "TargetingSetting",
        "TargetRestriction",
        "TargetRestrictionOperation",
    },
)


class TargetingSetting(proto.Message):
    r"""Settings for the targeting-related features, at the campaign
    and ad group levels. For more details about the targeting
    setting, visit
    https://support.google.com/google-ads/answer/7365594

    Attributes:
        target_restrictions (MutableSequence[google.ads.googleads.v22.common.types.TargetRestriction]):
            The per-targeting-dimension setting to
            restrict the reach of your campaign or ad group.
        target_restriction_operations (MutableSequence[google.ads.googleads.v22.common.types.TargetRestrictionOperation]):
            The list of operations changing the target
            restrictions.
            Adding a target restriction with a targeting
            dimension that already exists causes the
            existing target restriction to be replaced with
            the new value.
    """

    target_restrictions: MutableSequence["TargetRestriction"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="TargetRestriction",
        )
    )
    target_restriction_operations: MutableSequence[
        "TargetRestrictionOperation"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=2,
        message="TargetRestrictionOperation",
    )


class TargetRestriction(proto.Message):
    r"""The list of per-targeting-dimension targeting settings.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        targeting_dimension (google.ads.googleads.v22.enums.types.TargetingDimensionEnum.TargetingDimension):
            The targeting dimension that these settings
            apply to.
        bid_only (bool):
            Indicates whether to restrict your ads to show only for the
            criteria you have selected for this targeting_dimension, or
            to target all values for this targeting_dimension and show
            ads based on your targeting in other TargetingDimensions. A
            value of ``true`` means that these criteria will only apply
            bid modifiers, and not affect targeting. A value of
            ``false`` means that these criteria will restrict targeting
            as well as applying bid modifiers.

            This field is a member of `oneof`_ ``_bid_only``.
    """

    targeting_dimension: (
        gage_targeting_dimension.TargetingDimensionEnum.TargetingDimension
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=gage_targeting_dimension.TargetingDimensionEnum.TargetingDimension,
    )
    bid_only: bool = proto.Field(
        proto.BOOL,
        number=3,
        optional=True,
    )


class TargetRestrictionOperation(proto.Message):
    r"""Operation to be performed on a target restriction list in a
    mutate.

    Attributes:
        operator (google.ads.googleads.v22.common.types.TargetRestrictionOperation.Operator):
            Type of list operation to perform.
        value (google.ads.googleads.v22.common.types.TargetRestriction):
            The target restriction being added to or
            removed from the list.
    """

    class Operator(proto.Enum):
        r"""The operator.

        Values:
            UNSPECIFIED (0):
                Unspecified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            ADD (2):
                Add the restriction to the existing
                restrictions.
            REMOVE (3):
                Remove the restriction from the existing
                restrictions.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        ADD = 2
        REMOVE = 3

    operator: Operator = proto.Field(
        proto.ENUM,
        number=1,
        enum=Operator,
    )
    value: "TargetRestriction" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="TargetRestriction",
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "AdFormatTypeEnum",
    },
)


class AdFormatTypeEnum(proto.Message):
    r"""Container for enumeration of Google Ads format types."""

    class AdFormatType(proto.Enum):
        r"""Enumerates Google Ads format types.

        Note that this segmentation is available only for Video and
        Demand Gen campaigns. For assets, only video assets are
        supported.

        Values:
            UNSPECIFIED (0):
                No value has been specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            OTHER (2):
                Value assigned to formats (such as
                experimental formats) which don't support format
                segmentation in Video and Demand Gen campaigns.

                Note that these formats may change categories in
                the future, for example if an experimental
                format is exposed or a new format is added. We
                strongly recommend to not rely on this field for
                long term decisions.
            UNSEGMENTED (3):
                Value assigned for Video TrueView for Action
                campaigns statistics.
                Note that statistics with this value may change
                categories in the future, for example if format
                segmentation support is added for new campaign
                types. We strongly recommend to not rely on this
                field for long term decisions.
            INSTREAM_SKIPPABLE (4):
                Skippable in-stream ads.
            INSTREAM_NON_SKIPPABLE (5):
                Non-skippable in-stream ads.
            INFEED (6):
                In-feed YouTube or image ads served on feed
                surfaces (e.g. Discover Feed, YouTube Home,
                etc.).
            BUMPER (7):
                Short (<7 secs) in-stream non-skippable
                YouTube ads.
            OUTSTREAM (8):
                Outstream ads.
            MASTHEAD (9):
                Masthead ads.
            AUDIO (10):
                Audio ads.
            SHORTS (11):
                Vertical full-screen video or image ads
                served on YouTube Shorts or BrandConnect ads
                served as organic YouTube Shorts.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        OTHER = 2
        UNSEGMENTED = 3
        INSTREAM_SKIPPABLE = 4
        INSTREAM_NON_SKIPPABLE = 5
        INFEED = 6
        BUMPER = 7
        OUTSTREAM = 8
        MASTHEAD = 9
        AUDIO = 10
        SHORTS = 11


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.common.types import asset_usage


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "AdGroupAdAssetCombinationView",
    },
)


class AdGroupAdAssetCombinationView(proto.Message):
    r"""A view on the usage of ad group ad asset combination.
    Now we only support AdGroupAdAssetCombinationView for Responsive
    Search Ads, with more ad types planned for the future.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the ad group ad asset
            combination view. The combination ID is 128 bits long, where
            the upper 64 bits are stored in asset_combination_id_high,
            and the lower 64 bits are stored in
            asset_combination_id_low. AdGroupAd Asset Combination view
            resource names have the form:
            ``customers/{customer_id}/adGroupAdAssetCombinationViews/{AdGroupAd.ad_group_id}~{AdGroupAd.ad.ad_id}~{AssetCombination.asset_combination_id_low}~{AssetCombination.asset_combination_id_high}``
        served_assets (MutableSequence[google.ads.googleads.v22.common.types.AssetUsage]):
            Output only. Served assets.
        enabled (bool):
            Output only. The status between the asset
            combination and the latest version of the ad. If
            true, the asset combination is linked to the
            latest version of the ad. If false, it means the
            link once existed but has been removed and is no
            longer present in the latest version of the ad.

            This field is a member of `oneof`_ ``_enabled``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    served_assets: MutableSequence[asset_usage.AssetUsage] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message=asset_usage.AssetUsage,
        )
    )
    enabled: bool = proto.Field(
        proto.BOOL,
        number=3,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v19.common.types import tag_snippet
from google.ads.googleads.v19.enums.types import (
    attribution_model as gage_attribution_model,
)
from google.ads.googleads.v19.enums.types import conversion_action_category
from google.ads.googleads.v19.enums.types import conversion_action_counting_type
from google.ads.googleads.v19.enums.types import conversion_action_status
from google.ads.googleads.v19.enums.types import conversion_action_type
from google.ads.googleads.v19.enums.types import conversion_origin
from google.ads.googleads.v19.enums.types import (
    data_driven_model_status as gage_data_driven_model_status,
)
from google.ads.googleads.v19.enums.types import (
    mobile_app_vendor as gage_mobile_app_vendor,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.resources",
    marshal="google.ads.googleads.v19",
    manifest={
        "ConversionAction",
    },
)


class ConversionAction(proto.Message):
    r"""A conversion action.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the conversion action.
            Conversion action resource names have the form:

            ``customers/{customer_id}/conversionActions/{conversion_action_id}``
        id (int):
            Output only. The ID of the conversion action.

            This field is a member of `oneof`_ ``_id``.
        name (str):
            The name of the conversion action.

            This field is required and should not be empty
            when creating new conversion actions.

            This field is a member of `oneof`_ ``_name``.
        status (google.ads.googleads.v19.enums.types.ConversionActionStatusEnum.ConversionActionStatus):
            The status of this conversion action for
            conversion event accrual.
        type_ (google.ads.googleads.v19.enums.types.ConversionActionTypeEnum.ConversionActionType):
            Immutable. The type of this conversion
            action.
        origin (google.ads.googleads.v19.enums.types.ConversionOriginEnum.ConversionOrigin):
            Output only. The conversion origin of this
            conversion action.
        primary_for_goal (bool):
            If a conversion action's primary_for_goal bit is false, the
            conversion action is non-biddable for all campaigns
            regardless of their customer conversion goal or campaign
            conversion goal. However, custom conversion goals do not
            respect primary_for_goal, so if a campaign has a custom
            conversion goal configured with a primary_for_goal = false
            conversion action, that conversion action is still biddable.
            By default, primary_for_goal will be true if not set. In V9,
            primary_for_goal can only be set to false after creation
            through an 'update' operation because it's not declared as
            optional.

            This field is a member of `oneof`_ ``_primary_for_goal``.
        category (google.ads.googleads.v19.enums.types.ConversionActionCategoryEnum.ConversionActionCategory):
            The category of conversions reported for this
            conversion action.
        owner_customer (str):
            Output only. The resource name of the
            conversion action owner customer, or null if
            this is a system-defined conversion action.

            This field is a member of `oneof`_ ``_owner_customer``.
        include_in_conversions_metric (bool):
            Whether this conversion action should be
            included in the "conversions" metric.

            This field is a member of `oneof`_ ``_include_in_conversions_metric``.
        click_through_lookback_window_days (int):
            The maximum number of days that may elapse
            between an interaction (for example, a click)
            and a conversion event.

            This field is a member of `oneof`_ ``_click_through_lookback_window_days``.
        view_through_lookback_window_days (int):
            The maximum number of days which may elapse
            between an impression and a conversion without
            an interaction.

            This field is a member of `oneof`_ ``_view_through_lookback_window_days``.
        value_settings (google.ads.googleads.v19.resources.types.ConversionAction.ValueSettings):
            Settings related to the value for conversion
            events associated with this conversion action.
        counting_type (google.ads.googleads.v19.enums.types.ConversionActionCountingTypeEnum.ConversionActionCountingType):
            How to count conversion events for the
            conversion action.
        attribution_model_settings (google.ads.googleads.v19.resources.types.ConversionAction.AttributionModelSettings):
            Settings related to this conversion action's
            attribution model.
        tag_snippets (MutableSequence[google.ads.googleads.v19.common.types.TagSnippet]):
            Output only. The snippets used for tracking
            conversions.
        phone_call_duration_seconds (int):
            The phone call duration in seconds after
            which a conversion should be reported for this
            conversion action.

            The value must be between 0 and 10000,
            inclusive.

            This field is a member of `oneof`_ ``_phone_call_duration_seconds``.
        app_id (str):
            App ID for an app conversion action.

            This field is a member of `oneof`_ ``_app_id``.
        mobile_app_vendor (google.ads.googleads.v19.enums.types.MobileAppVendorEnum.MobileAppVendor):
            Output only. Mobile app vendor for an app
            conversion action.
        firebase_settings (google.ads.googleads.v19.resources.types.ConversionAction.FirebaseSettings):
            Output only. Firebase settings for Firebase
            conversion types.
        third_party_app_analytics_settings (google.ads.googleads.v19.resources.types.ConversionAction.ThirdPartyAppAnalyticsSettings):
            Output only. Third Party App Analytics
            settings for third party conversion types.
        google_analytics_4_settings (google.ads.googleads.v19.resources.types.ConversionAction.GoogleAnalytics4Settings):
            Output only. Google Analytics 4 settings for
            Google Analytics 4 conversion types.
    """

    class AttributionModelSettings(proto.Message):
        r"""Settings related to this conversion action's attribution
        model.

        Attributes:
            attribution_model (google.ads.googleads.v19.enums.types.AttributionModelEnum.AttributionModel):
                The attribution model type of this conversion
                action.
            data_driven_model_status (google.ads.googleads.v19.enums.types.DataDrivenModelStatusEnum.DataDrivenModelStatus):
                Output only. The status of the data-driven
                attribution model for the conversion action.
        """

        attribution_model: (
            gage_attribution_model.AttributionModelEnum.AttributionModel
        ) = proto.Field(
            proto.ENUM,
            number=1,
            enum=gage_attribution_model.AttributionModelEnum.AttributionModel,
        )
        data_driven_model_status: (
            gage_data_driven_model_status.DataDrivenModelStatusEnum.DataDrivenModelStatus
        ) = proto.Field(
            proto.ENUM,
            number=2,
            enum=gage_data_driven_model_status.DataDrivenModelStatusEnum.DataDrivenModelStatus,
        )

    class ValueSettings(proto.Message):
        r"""Settings related to the value for conversion events
        associated with this conversion action.


        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            default_value (float):
                The value to use when conversion events for
                this conversion action are sent with an invalid,
                disallowed or missing value, or when this
                conversion action is configured to always use
                the default value.

                This field is a member of `oneof`_ ``_default_value``.
            default_currency_code (str):
                The currency code to use when conversion
                events for this conversion action are sent with
                an invalid or missing currency code, or when
                this conversion action is configured to always
                use the default value.

                This field is a member of `oneof`_ ``_default_currency_code``.
            always_use_default_value (bool):
                Controls whether the default value and
                default currency code are used in place of the
                value and currency code specified in conversion
                events for this conversion action.

                This field is a member of `oneof`_ ``_always_use_default_value``.
        """

        default_value: float = proto.Field(
            proto.DOUBLE,
            number=4,
            optional=True,
        )
        default_currency_code: str = proto.Field(
            proto.STRING,
            number=5,
            optional=True,
        )
        always_use_default_value: bool = proto.Field(
            proto.BOOL,
            number=6,
            optional=True,
        )

    class ThirdPartyAppAnalyticsSettings(proto.Message):
        r"""Settings related to a third party app analytics conversion
        action.


        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            event_name (str):
                Output only. The event name of a third-party
                app analytics conversion.

                This field is a member of `oneof`_ ``_event_name``.
            provider_name (str):
                Output only. Name of the third-party app
                analytics provider.
        """

        event_name: str = proto.Field(
            proto.STRING,
            number=2,
            optional=True,
        )
        provider_name: str = proto.Field(
            proto.STRING,
            number=3,
        )

    class FirebaseSettings(proto.Message):
        r"""Settings related to a Firebase conversion action.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            event_name (str):
                Output only. The event name of a Firebase
                conversion.

                This field is a member of `oneof`_ ``_event_name``.
            project_id (str):
                Output only. The Firebase project ID of the
                conversion.

                This field is a member of `oneof`_ ``_project_id``.
            property_id (int):
                Output only. The GA property ID of the
                conversion.
            property_name (str):
                Output only. The GA property name of the
                conversion.
        """

        event_name: str = proto.Field(
            proto.STRING,
            number=3,
            optional=True,
        )
        project_id: str = proto.Field(
            proto.STRING,
            number=4,
            optional=True,
        )
        property_id: int = proto.Field(
            proto.INT64,
            number=5,
        )
        property_name: str = proto.Field(
            proto.STRING,
            number=6,
        )

    class GoogleAnalytics4Settings(proto.Message):
        r"""Settings related to a Google Analytics 4 conversion action.

        Attributes:
            event_name (str):
                Output only. The name of the GA 4 event.
            property_name (str):
                Output only. The name of the GA 4 property.
            property_id (int):
                Output only. The ID of the GA 4 property.
        """

        event_name: str = proto.Field(
            proto.STRING,
            number=1,
        )
        property_name: str = proto.Field(
            proto.STRING,
            number=2,
        )
        property_id: int = proto.Field(
            proto.INT64,
            number=3,
        )

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=21,
        optional=True,
    )
    name: str = proto.Field(
        proto.STRING,
        number=22,
        optional=True,
    )
    status: (
        conversion_action_status.ConversionActionStatusEnum.ConversionActionStatus
    ) = proto.Field(
        proto.ENUM,
        number=4,
        enum=conversion_action_status.ConversionActionStatusEnum.ConversionActionStatus,
    )
    type_: (
        conversion_action_type.ConversionActionTypeEnum.ConversionActionType
    ) = proto.Field(
        proto.ENUM,
        number=5,
        enum=conversion_action_type.ConversionActionTypeEnum.ConversionActionType,
    )
    origin: conversion_origin.ConversionOriginEnum.ConversionOrigin = (
        proto.Field(
            proto.ENUM,
            number=30,
            enum=conversion_origin.ConversionOriginEnum.ConversionOrigin,
        )
    )
    primary_for_goal: bool = proto.Field(
        proto.BOOL,
        number=31,
        optional=True,
    )
    category: (
        conversion_action_category.ConversionActionCategoryEnum.ConversionActionCategory
    ) = proto.Field(
        proto.ENUM,
        number=6,
        enum=conversion_action_category.ConversionActionCategoryEnum.ConversionActionCategory,
    )
    owner_customer: str = proto.Field(
        proto.STRING,
        number=23,
        optional=True,
    )
    include_in_conversions_metric: bool = proto.Field(
        proto.BOOL,
        number=24,
        optional=True,
    )
    click_through_lookback_window_days: int = proto.Field(
        proto.INT64,
        number=25,
        optional=True,
    )
    view_through_lookback_window_days: int = proto.Field(
        proto.INT64,
        number=26,
        optional=True,
    )
    value_settings: ValueSettings = proto.Field(
        proto.MESSAGE,
        number=11,
        message=ValueSettings,
    )
    counting_type: (
        conversion_action_counting_type.ConversionActionCountingTypeEnum.ConversionActionCountingType
    ) = proto.Field(
        proto.ENUM,
        number=12,
        enum=conversion_action_counting_type.ConversionActionCountingTypeEnum.ConversionActionCountingType,
    )
    attribution_model_settings: AttributionModelSettings = proto.Field(
        proto.MESSAGE,
        number=13,
        message=AttributionModelSettings,
    )
    tag_snippets: MutableSequence[tag_snippet.TagSnippet] = proto.RepeatedField(
        proto.MESSAGE,
        number=14,
        message=tag_snippet.TagSnippet,
    )
    phone_call_duration_seconds: int = proto.Field(
        proto.INT64,
        number=27,
        optional=True,
    )
    app_id: str = proto.Field(
        proto.STRING,
        number=28,
        optional=True,
    )
    mobile_app_vendor: (
        gage_mobile_app_vendor.MobileAppVendorEnum.MobileAppVendor
    ) = proto.Field(
        proto.ENUM,
        number=17,
        enum=gage_mobile_app_vendor.MobileAppVendorEnum.MobileAppVendor,
    )
    firebase_settings: FirebaseSettings = proto.Field(
        proto.MESSAGE,
        number=18,
        message=FirebaseSettings,
    )
    third_party_app_analytics_settings: ThirdPartyAppAnalyticsSettings = (
        proto.Field(
            proto.MESSAGE,
            number=19,
            message=ThirdPartyAppAnalyticsSettings,
        )
    )
    google_analytics_4_settings: GoogleAnalytics4Settings = proto.Field(
        proto.MESSAGE,
        number=34,
        message=GoogleAnalytics4Settings,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v22.enums.types import (
    placement_type as gage_placement_type,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "GroupContentSuitabilityPlacementView",
    },
)


class GroupContentSuitabilityPlacementView(proto.Message):
    r"""A group content suitability placement view.

    Attributes:
        resource_name (str):
            Output only. The resource name of the group content
            suitability placement view. Group content suitability
            placement view resource names have the form:

            ``customers/{customer_id}/groupContentSuitabilityPlacementViews/{placement_fingerprint}``
        display_name (str):
            Output only. The display name is URL for
            websites, YouTube video name for YouTube videos,
            and translated mobile app name for mobile apps.
        placement (str):
            Output only. The automatic placement string
            at group level, for example. website url, mobile
            application id, or a YouTube video id.
        placement_type (google.ads.googleads.v22.enums.types.PlacementTypeEnum.PlacementType):
            Output only. Represents the type of the
            placement, for example, Website, YouTubeVideo
            and MobileApplication.
        target_url (str):
            Output only. URL of the placement, for
            example, website, link to the mobile application
            in app store, or a YouTube video URL.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    placement: str = proto.Field(
        proto.STRING,
        number=3,
    )
    placement_type: gage_placement_type.PlacementTypeEnum.PlacementType = (
        proto.Field(
            proto.ENUM,
            number=4,
            enum=gage_placement_type.PlacementTypeEnum.PlacementType,
        )
    )
    target_url: str = proto.Field(
        proto.STRING,
        number=5,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v22.common.types import customizer_value
from google.ads.googleads.v22.enums.types import customizer_value_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "CampaignCustomizer",
    },
)


class CampaignCustomizer(proto.Message):
    r"""A customizer value for the associated CustomizerAttribute at
    the Campaign level.

    Attributes:
        resource_name (str):
            Immutable. The resource name of the campaign customizer.
            Campaign customizer resource names have the form:

            ``customers/{customer_id}/campaignCustomizers/{campaign_id}~{customizer_attribute_id}``
        campaign (str):
            Immutable. The campaign to which the
            customizer attribute is linked.
        customizer_attribute (str):
            Required. Immutable. The customizer attribute
            which is linked to the campaign.
        status (google.ads.googleads.v22.enums.types.CustomizerValueStatusEnum.CustomizerValueStatus):
            Output only. The status of the campaign
            customizer.
        value (google.ads.googleads.v22.common.types.CustomizerValue):
            Required. The value to associate with the
            customizer attribute at this level. The value
            must be of the type specified for the
            CustomizerAttribute.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    campaign: str = proto.Field(
        proto.STRING,
        number=2,
    )
    customizer_attribute: str = proto.Field(
        proto.STRING,
        number=3,
    )
    status: (
        customizer_value_status.CustomizerValueStatusEnum.CustomizerValueStatus
    ) = proto.Field(
        proto.ENUM,
        number=4,
        enum=customizer_value_status.CustomizerValueStatusEnum.CustomizerValueStatus,
    )
    value: customizer_value.CustomizerValue = proto.Field(
        proto.MESSAGE,
        number=5,
        message=customizer_value.CustomizerValue,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.errors",
    marshal="google.ads.googleads.v21",
    manifest={
        "FeedErrorEnum",
    },
)


class FeedErrorEnum(proto.Message):
    r"""Container for enum describing possible feed errors."""

    class FeedError(proto.Enum):
        r"""Enum describing possible feed errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            ATTRIBUTE_NAMES_NOT_UNIQUE (2):
                The names of the FeedAttributes must be
                unique.
            ATTRIBUTES_DO_NOT_MATCH_EXISTING_ATTRIBUTES (3):
                The attribute list must be an exact copy of
                the existing list if the attribute ID's are
                present.
            CANNOT_SPECIFY_USER_ORIGIN_FOR_SYSTEM_FEED (4):
                Cannot specify USER origin for a system
                generated feed.
            CANNOT_SPECIFY_GOOGLE_ORIGIN_FOR_NON_SYSTEM_FEED (5):
                Cannot specify GOOGLE origin for a non-system
                generated feed.
            CANNOT_SPECIFY_FEED_ATTRIBUTES_FOR_SYSTEM_FEED (6):
                Cannot specify feed attributes for system
                feed.
            CANNOT_UPDATE_FEED_ATTRIBUTES_WITH_ORIGIN_GOOGLE (7):
                Cannot update FeedAttributes on feed with
                origin GOOGLE.
            FEED_REMOVED (8):
                The given ID refers to a removed Feed.
                Removed Feeds are immutable.
            INVALID_ORIGIN_VALUE (9):
                The origin of the feed is not valid for the
                client.
            FEED_ORIGIN_IS_NOT_USER (10):
                A user can only create and modify feeds with
                USER origin.
            INVALID_AUTH_TOKEN_FOR_EMAIL (11):
                Invalid auth token for the given email.
            INVALID_EMAIL (12):
                Invalid email specified.
            DUPLICATE_FEED_NAME (13):
                Feed name matches that of another active
                Feed.
            INVALID_FEED_NAME (14):
                Name of feed is not allowed.
            MISSING_OAUTH_INFO (15):
                Missing OAuthInfo.
            NEW_ATTRIBUTE_CANNOT_BE_PART_OF_UNIQUE_KEY (16):
                New FeedAttributes must not affect the unique
                key.
            TOO_MANY_ATTRIBUTES (17):
                Too many FeedAttributes for a Feed.
            INVALID_BUSINESS_ACCOUNT (18):
                The business account is not valid.
            BUSINESS_ACCOUNT_CANNOT_ACCESS_LOCATION_ACCOUNT (19):
                Business account cannot access Business
                Profile.
            INVALID_AFFILIATE_CHAIN_ID (20):
                Invalid chain ID provided for affiliate
                location feed.
            DUPLICATE_SYSTEM_FEED (21):
                There is already a feed with the given system
                feed generation data.
            GMB_ACCESS_ERROR (22):
                An error occurred accessing Business Profile.
            CANNOT_HAVE_LOCATION_AND_AFFILIATE_LOCATION_FEEDS (23):
                A customer cannot have both LOCATION and AFFILIATE_LOCATION
                feeds.
            LEGACY_EXTENSION_TYPE_READ_ONLY (24):
                Feed-based extension is read-only for this
                extension type.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        ATTRIBUTE_NAMES_NOT_UNIQUE = 2
        ATTRIBUTES_DO_NOT_MATCH_EXISTING_ATTRIBUTES = 3
        CANNOT_SPECIFY_USER_ORIGIN_FOR_SYSTEM_FEED = 4
        CANNOT_SPECIFY_GOOGLE_ORIGIN_FOR_NON_SYSTEM_FEED = 5
        CANNOT_SPECIFY_FEED_ATTRIBUTES_FOR_SYSTEM_FEED = 6
        CANNOT_UPDATE_FEED_ATTRIBUTES_WITH_ORIGIN_GOOGLE = 7
        FEED_REMOVED = 8
        INVALID_ORIGIN_VALUE = 9
        FEED_ORIGIN_IS_NOT_USER = 10
        INVALID_AUTH_TOKEN_FOR_EMAIL = 11
        INVALID_EMAIL = 12
        DUPLICATE_FEED_NAME = 13
        INVALID_FEED_NAME = 14
        MISSING_OAUTH_INFO = 15
        NEW_ATTRIBUTE_CANNOT_BE_PART_OF_UNIQUE_KEY = 16
        TOO_MANY_ATTRIBUTES = 17
        INVALID_BUSINESS_ACCOUNT = 18
        BUSINESS_ACCOUNT_CANNOT_ACCESS_LOCATION_ACCOUNT = 19
        INVALID_AFFILIATE_CHAIN_ID = 20
        DUPLICATE_SYSTEM_FEED = 21
        GMB_ACCESS_ERROR = 22
        CANNOT_HAVE_LOCATION_AND_AFFILIATE_LOCATION_FEEDS = 23
        LEGACY_EXTENSION_TYPE_READ_ONLY = 24


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.enums",
    marshal="google.ads.googleads.v22",
    manifest={
        "AssetGroupSignalApprovalStatusEnum",
    },
)


class AssetGroupSignalApprovalStatusEnum(proto.Message):
    r"""Container for enum describing possible AssetGroupSignal
    approval statuses. Details see
    https://support.google.com/google-ads/answer/2453978.

    """

    class AssetGroupSignalApprovalStatus(proto.Enum):
        r"""Enumerates AssetGroupSignal approval statuses, which are only
        used for Search Theme Signal.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                The value is unknown in this version.
            APPROVED (2):
                Search Theme is eligible to show ads.
            LIMITED (3):
                Low search volume; Below first page bid
                estimate.
            DISAPPROVED (4):
                Search Theme is inactive and isn't showing
                ads. A disapproved Search Theme usually means
                there's an issue with one or more of our
                advertising policies.
            UNDER_REVIEW (5):
                Search Theme is under review. It won’t be
                able to trigger ads until it's been reviewed.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        APPROVED = 2
        LIMITED = 3
        DISAPPROVED = 4
        UNDER_REVIEW = 5


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.errors",
    marshal="google.ads.googleads.v21",
    manifest={
        "ConversionGoalCampaignConfigErrorEnum",
    },
)


class ConversionGoalCampaignConfigErrorEnum(proto.Message):
    r"""Container for enum describing possible conversion goal
    campaign config errors.

    """

    class ConversionGoalCampaignConfigError(proto.Enum):
        r"""Enum describing possible conversion goal campaign config
        errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            CANNOT_USE_CAMPAIGN_GOAL_FOR_SEARCH_ADS_360_MANAGED_CAMPAIGN (2):
                Campaign is managed by Search Ads 360 but
                uses Unified Goal.
            CUSTOM_GOAL_DOES_NOT_BELONG_TO_GOOGLE_ADS_CONVERSION_CUSTOMER (3):
                The campaign is using a custom goal that does
                not belong to its Google Ads conversion customer
                (conversion tracking customer).
            CAMPAIGN_CANNOT_USE_UNIFIED_GOALS (4):
                The campaign is not allowed to use unified
                goals.
            EMPTY_CONVERSION_GOALS (5):
                The campaign is using campaign override goals
                but has no goals configured.
            STORE_SALE_STORE_VISIT_CANNOT_BE_BOTH_INCLUDED (6):
                STORE_SALE and STORE_VISIT conversion types cannot be both
                included in campaign level goal.
            PERFORMANCE_MAX_CAMPAIGN_CANNOT_USE_CUSTOM_GOAL_WITH_STORE_SALES (7):
                Performance Max campaign is not allowed to
                use custom goal with store sales conversion
                type.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        CANNOT_USE_CAMPAIGN_GOAL_FOR_SEARCH_ADS_360_MANAGED_CAMPAIGN = 2
        CUSTOM_GOAL_DOES_NOT_BELONG_TO_GOOGLE_ADS_CONVERSION_CUSTOMER = 3
        CAMPAIGN_CANNOT_USE_UNIFIED_GOALS = 4
        EMPTY_CONVERSION_GOALS = 5
        STORE_SALE_STORE_VISIT_CANNOT_BE_BOTH_INCLUDED = 6
        PERFORMANCE_MAX_CAMPAIGN_CANNOT_USE_CUSTOM_GOAL_WITH_STORE_SALES = 7


__all__ = tuple(sorted(__protobuf__.manifest))
